package com.justep.cordova.plugin.filebrowser;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.res.Resources;
import android.graphics.drawable.ColorDrawable;
import android.net.Uri;
import android.provider.MediaStore;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.PopupWindow;

import com.justep.filebrowser.FolderManagerActivity;
import com.justep.filebrowser.SelectActivity;
import com.justep.filebrowser.global.FilePath;
import com.justep.filebrowser.R;
import com.lzy.imagepicker.ImagePicker;
import com.lzy.imagepicker.loader.GlideImageLoader;
import com.lzy.imagepicker.ui.ImageGridActivity;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.PluginResult;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import static android.app.Activity.RESULT_CANCELED;


public class FileBrowser extends CordovaPlugin {

  private PopupWindow fileChooserPopup;
  private CallbackContext callbackContext;

  private static final int VIDEOCHOOSER_RESULTCODE = 1001;
  private static final int IMAGECHOOSER_RESULTCODE = 1002;
  private static final int AUDIOCHOOSER_RESULTCODE = 1003;

  Uri fileUri;

  Button btn_camera_photo;
  Button btn_camera_video;
  Button btn_record_audio;
  Button btn_photos;
  Button btn_file;

  View stroke_camera_photo;
  View stroke_camera_video;
  View stroke_record_audio;
  View stroke_photos;
  View stroke_file;

  List<String> acceptType = new ArrayList<String>();
  String mExitCaption;
  int mSelectedMax;

  private static List<String> fromString(String string) {
    if(string == null || string.isEmpty()){
      List temp = new ArrayList<String>();
      temp.add("*/*");
      return temp;
    }
    String[] result = string.split(";");
    List<String> types =  Arrays.asList(result);
    return types;
  }


  //获取是否存在NavigationBar
  public static boolean checkDeviceHasNavigationBar(Context context) {
    boolean hasNavigationBar = false;
    Resources rs = context.getResources();
    int id = rs.getIdentifier("config_showNavigationBar", "bool", "android");
    if (id > 0) {
      hasNavigationBar = rs.getBoolean(id);
    }
    try {
      Class systemPropertiesClass = Class.forName("android.os.SystemProperties");
      Method m = systemPropertiesClass.getMethod("get", String.class);
      String navBarOverride = (String) m.invoke(systemPropertiesClass, "qemu.hw.mainkeys");
      if ("1".equals(navBarOverride)) {
        hasNavigationBar = false;
      } else if ("0".equals(navBarOverride)) {
        hasNavigationBar = true;
      }
    } catch (Exception e) {

    }
    return hasNavigationBar;

  }

  public void showFileUploadChooserPopup() {

    btn_camera_photo.setVisibility(View.GONE);
    stroke_camera_photo.setVisibility(View.GONE);
    btn_camera_video.setVisibility(View.GONE);
    stroke_camera_video.setVisibility(View.GONE);
    btn_record_audio.setVisibility(View.GONE);
    stroke_record_audio.setVisibility(View.GONE);
    btn_photos.setVisibility(View.GONE);
    stroke_photos.setVisibility(View.GONE);
    btn_file.setVisibility(View.GONE);
    stroke_file.setVisibility(View.GONE);


    for(int i = 0; i < acceptType.size(); i++) {
      if (acceptType.get(i).equals("image/*")) {
        if (i + 1 < acceptType.size() && acceptType.get(i + 1).equals("doc/-")) {
          visibilityButtons.add(btn_camera_photo);
          btn_camera_photo.setVisibility(View.VISIBLE);
          stroke_camera_photo.setVisibility(View.VISIBLE);
          i++;
        } else {
          visibilityButtons.add(btn_photos);
          btn_photos.setVisibility(View.VISIBLE);
          stroke_photos.setVisibility(View.VISIBLE);
        }
        continue;
      }

      if (acceptType.get(i).equals("video/*")) {
        if (i + 1 < acceptType.size() && acceptType.get(i + 1).equals("doc/-")) {
          visibilityButtons.add(btn_camera_video);
          btn_camera_video.setVisibility(View.VISIBLE);
          stroke_camera_video.setVisibility(View.VISIBLE);
          i++;
        } else {
          visibilityButtons.add(btn_photos);
          btn_photos.setVisibility(View.VISIBLE);
          stroke_photos.setVisibility(View.VISIBLE);
        }
        continue;
      }

      if (acceptType.get(i).equals("audio/*")) {
        btn_record_audio.setVisibility(View.VISIBLE);
        stroke_record_audio.setVisibility(View.VISIBLE);
        continue;
      }

      if (acceptType.get(i).equals("*/*")) {
        btn_file.setVisibility(View.VISIBLE);
        stroke_file.setVisibility(View.VISIBLE);
        continue;
      }
    }

    this.checkAutoClick();

    makeWindowDark();
    if(visibilityButtons.size() > 1) {
      Boolean HasNavigationBar = checkDeviceHasNavigationBar(cordova.getActivity());
      if (HasNavigationBar) {
        fileChooserPopup.showAtLocation(webView.getView(), Gravity.BOTTOM, 0, 100);
      } else {
        fileChooserPopup.showAtLocation(webView.getView(), Gravity.BOTTOM, 0, 0);
      }
    }
  }

  public void cancelFileChooserPopup(){
    fileChooserPopup.dismiss();
    Intent intent = new Intent();
    cordova.getActivity().setResult(RESULT_CANCELED, intent);
  };

  public void makeWindowDark() {
    Window window = cordova.getActivity().getWindow();
    WindowManager.LayoutParams lp = window.getAttributes();
    lp.alpha = 0.5f;
    window.setAttributes(lp);
  }

  public void makeWindowLight() {
    Window window = cordova.getActivity().getWindow();
    WindowManager.LayoutParams lp = window.getAttributes();
    lp.alpha = 1f;
    window.setAttributes(lp);
  }

  private void initFileUploadChooserPopup() {
    final CordovaPlugin plugin = this;
    LayoutInflater inflater = LayoutInflater.from(cordova.getActivity());
    View view = inflater.inflate(com.justep.filebrowser.R.layout.filebrowser_chooser, null);
    //当view参数为null时候，不实例化PopupWindow对象，实例代码：
    if(view != null) {
      fileChooserPopup = new PopupWindow(view, webView.getView().getWidth() - 20, ViewGroup.LayoutParams.WRAP_CONTENT, true);
      fileChooserPopup.setFocusable(true);
      fileChooserPopup.setBackgroundDrawable(new ColorDrawable());
      fileChooserPopup.setOutsideTouchable(true);
      fileChooserPopup.setAnimationStyle(com.justep.filebrowser.R.style.PopupWindowAnimation);
    }

    btn_camera_photo = (Button) view.findViewById(com.justep.filebrowser.R.id.btn_camera_photo);
    stroke_camera_photo = (View) view.findViewById(com.justep.filebrowser.R.id.stroke_camera_photo);
    btn_camera_photo.setOnClickListener(new View.OnClickListener() {

      private Uri getOutputMediaUri() {
        String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());
        File mediaFile = null;
        mediaFile = new File(FilePath.ROOT_DIR + File.separator +
          "IMG_" + timeStamp + ".jpg");
        return Uri.fromFile(mediaFile);
      }

      @Override
      public void onClick(View v) {
        fileUri = null;
        fileChooserPopup.dismiss();

        Intent intent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);//create a intent to take picture
        fileUri = getOutputMediaUri();
        intent.putExtra(MediaStore.EXTRA_OUTPUT, fileUri); // set the image file name
        cordova.startActivityForResult(plugin, intent, IMAGECHOOSER_RESULTCODE);
        cordova.getActivity().overridePendingTransition(R.anim.push_left_in, R.anim.push_left_out);
      }
    });

    btn_camera_video = (Button) view.findViewById(com.justep.filebrowser.R.id.btn_camera_video);
    stroke_camera_video= (View) view.findViewById(com.justep.filebrowser.R.id.stroke_camera_video);
    btn_camera_video.setOnClickListener(new View.OnClickListener() {

	    private Uri getOutputMediaUri() {
		    String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());
		    File mediaFile = null;
		    mediaFile = new File(FilePath.ROOT_DIR + File.separator +
			    "VIDEO_" + timeStamp + ".mp4");
		    return Uri.fromFile(mediaFile);
	    }

      @Override
      public void onClick(View v) {
	      fileUri = null;
        fileChooserPopup.dismiss();

        Intent intent = new Intent(MediaStore.ACTION_VIDEO_CAPTURE);
        fileUri = getOutputMediaUri();
        // set the video file name
        intent.putExtra(MediaStore.EXTRA_OUTPUT, fileUri);

        // set the video quality high
        intent.putExtra(MediaStore.EXTRA_VIDEO_QUALITY, 1);
	      //设置视频最大允许录制的时长，单位为毫秒。
        intent.putExtra(MediaStore.EXTRA_DURATION_LIMIT, 30000);
	      //指定视频最大允许的尺寸，单位为byte。
        intent.putExtra(MediaStore.EXTRA_SIZE_LIMIT, 20 * 1024 * 1024);

        cordova.startActivityForResult(plugin, intent, VIDEOCHOOSER_RESULTCODE);
        cordova.getActivity().overridePendingTransition(R.anim.push_left_in, R.anim.push_left_out);
      }
    });

    btn_record_audio = (Button) view.findViewById(com.justep.filebrowser.R.id.btn_record_audio);
    stroke_record_audio= (View) view.findViewById(com.justep.filebrowser.R.id.stroke_record_audio);
    btn_record_audio.setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        fileChooserPopup.dismiss();
	      Intent intent = new Intent(MediaStore.Audio.Media.RECORD_SOUND_ACTION);
	      cordova.startActivityForResult(plugin, intent, AUDIOCHOOSER_RESULTCODE);
        cordova.getActivity().overridePendingTransition(R.anim.push_left_in, R.anim.push_left_out);
      }
    });

    btn_photos = (Button) view.findViewById(R.id.btn_fbphotos);
    stroke_photos = (View) view.findViewById(R.id.stroke_fbphotos);
    btn_photos.setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        fileChooserPopup.dismiss();
        ImagePicker imagePicker;
        imagePicker = ImagePicker.getInstance();
        imagePicker.setImageLoader(new GlideImageLoader());

        imagePicker.setShowCamera(true);  //显示拍照按钮
        imagePicker.setCrop(false);        //允许裁剪（单选才有效）
        imagePicker.setSaveRectangle(false); //是否按矩形区域保存
        imagePicker.setSelectLimit(mSelectedMax);    //选中数量限制
        imagePicker.setFocusWidth(800);   //裁剪框的宽度。单位像素（圆形自动取宽高最小值）
        imagePicker.setFocusHeight(600);  //裁剪框的高度。单位像素（圆形自动取宽高最小值）
        imagePicker.setOutPutX(800);//保存文件的宽度。单位像素
        imagePicker.setOutPutY(600);//保存文件的高度。单位像素

        Intent intent = new Intent();
        intent.putExtra("exitCaption",  mExitCaption);
        intent.setClass(cordova.getActivity(), ImageGridActivity.class);
        cordova.startActivityForResult(plugin, intent, 100);
        cordova.getActivity().overridePendingTransition(R.anim.push_left_in, R.anim.push_left_out);
      }
    });

    btn_file = (Button) view.findViewById(com.justep.filebrowser.R.id.btn_file);
    stroke_file = (View) view.findViewById(com.justep.filebrowser.R.id.stroke_file);
    btn_file.setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        fileChooserPopup.dismiss();
        Intent intent = new Intent();
        intent.putExtra("maxSelectCount",  mSelectedMax);
        intent.putExtra("exitCaption",  mExitCaption);
        intent.setClass(cordova.getActivity(), SelectActivity.class);
        cordova.startActivityForResult(plugin, intent, 100);
        cordova.getActivity().overridePendingTransition(R.anim.push_left_in, R.anim.push_left_out);
      }
    });

    Button btn_cancel = (Button) view.findViewById(com.justep.filebrowser.R.id.btn_cancel);
    fileChooserPopup.setOnDismissListener(new PopupWindow.OnDismissListener() {
      @Override
      public void onDismiss() {
        makeWindowLight();
      }
    });

    btn_cancel.setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        cancelFileChooserPopup();
      }
    });
  }


  List<Button> visibilityButtons = new ArrayList<Button>();
  private void checkAutoClick() {
    if(visibilityButtons.size() == 1){
      webView.getView().post(new Runnable() {
        @Override
        public void run() {
          visibilityButtons.get(0).performClick();
        }
      });
    }
  }

  @Override
  public void pluginInitialize() {
	  FilePath.initRootDir(cordova.getActivity());
    initFileUploadChooserPopup();
  }

  @Override
  public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
    this.callbackContext = callbackContext;
    if("open".equals(action)) {
      cordova.getActivity().runOnUiThread(new Runnable() {
        public void run() {
          Intent intent = new Intent();
          intent.setClass(cordova.getActivity(), FolderManagerActivity.class);
          cordova.getActivity().startActivity(intent);
          cordova.getActivity().overridePendingTransition(R.anim.push_left_in, R.anim.push_left_out);
        }
      });

      // Don't return any result now
      PluginResult pluginResult = new PluginResult(PluginResult.Status.NO_RESULT);
      pluginResult.setKeepCallback(true);
      callbackContext.sendPluginResult(pluginResult);
      this.callbackContext = null;
      return true;
    }

    if ("select".equals(action)) {

      JSONObject params;
      params = args.getJSONObject(0);
      if(params.has("type")) {
        acceptType = fromString(params.getString("type"));
      }

      mExitCaption = "确定";
      if(params.has("exitCaption")) {
        mExitCaption = params.getString("exitCaption");
      }

      mSelectedMax = 9;
      if(params.has("maxSelectCount")) {
        mSelectedMax = params.getInt("maxSelectCount");
      }
      cordova.getActivity().runOnUiThread(new Runnable() {
        public void run() {
          showFileUploadChooserPopup();
        }
      });
      return true;
    }

    if("selectPicture".equals(action)){

      final CordovaPlugin plugin = this;
      cordova.getActivity().runOnUiThread(new Runnable() {
        public void run() {
          ImagePicker imagePicker;
          imagePicker = ImagePicker.getInstance();
          imagePicker.setImageLoader(new GlideImageLoader());

          imagePicker.setShowCamera(true);  //显示拍照按钮
          imagePicker.setCrop(false);        //允许裁剪（单选才有效）
          imagePicker.setSaveRectangle(false); //是否按矩形区域保存
          imagePicker.setSelectLimit(9);    //选中数量限制
          imagePicker.setFocusWidth(800);   //裁剪框的宽度。单位像素（圆形自动取宽高最小值）
          imagePicker.setFocusHeight(800);  //裁剪框的高度。单位像素（圆形自动取宽高最小值）
          imagePicker.setOutPutX(1000);//保存文件的宽度。单位像素
          imagePicker.setOutPutY(1000);//保存文件的高度。单位像素

          Intent intent = new Intent();
          intent.setClass(cordova.getActivity(), ImageGridActivity.class);
          cordova.startActivityForResult(plugin, intent, 100);
          cordova.getActivity().overridePendingTransition(R.anim.push_left_in, R.anim.push_left_out);
        }
      });
    }
    return true;
  }
  
    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
      super.onActivityResult(requestCode, resultCode, data);
	    if(requestCode == VIDEOCHOOSER_RESULTCODE || requestCode == IMAGECHOOSER_RESULTCODE){
				if(resultCode != 0){
					ArrayList<String> filenames = new ArrayList<String>();
					filenames.add(fileUri.toString());
					JSONArray res = new JSONArray(filenames);
					this.callbackContext.success(res);
					return;
				}

				JSONArray res = new JSONArray();
				this.callbackContext.success(res);
				return;
	    }

	    if(requestCode == AUDIOCHOOSER_RESULTCODE){
		    if(resultCode == 0 || data == null) {
			    JSONArray res = new JSONArray();
			    this.callbackContext.success(res);
			    return;
		    }

		    try {
			    Uri uri = data.getData();
			    //Cursor cursor = cordova.getActivity().getContentResolver().query(uri, null, null, null, null);
			    //cursor.moveToFirst();
			    //int index = cursor.getColumnIndex(MediaStore.Audio.AudioColumns.DATA);
			    //String filePath = cursor.getString(index);

          //Uri.fromPath(filePath);
			    //Log.d(TAG, filePath);
			    ArrayList<String> filenames = new ArrayList<String>();
			    filenames.add(uri.toString());
			    JSONArray res = new JSONArray(filenames);
			    this.callbackContext.success(res);
			    return;
		    }catch (Exception e) {
			    throw new RuntimeException(e);
		    }
	    }

      if (resultCode == Activity.RESULT_OK && data != null) {
        ArrayList<String> fileNames = data.getStringArrayListExtra("selectedfiles");
        ArrayList<String> Urifiles = new ArrayList<String>();
        for(String pathname:fileNames){
          Urifiles.add(Uri.fromFile(new File(pathname)).toString());
        }

        JSONArray res = new JSONArray(Urifiles);
        this.callbackContext.success(res);
      } else if (resultCode == RESULT_CANCELED && data != null) {
        String error = data.getStringExtra("ERRORMESSAGE");
        this.callbackContext.error(error);
      } else if (resultCode == RESULT_CANCELED) {
        JSONArray res = new JSONArray();
        this.callbackContext.success(res);
      } else {
        this.callbackContext.error("No images selected");
      }
    }
}