package com.justep.filebrowser;

/**
 * Created by lihong on 16-10-12.
 */

import android.app.Activity;
import android.content.Intent;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.justep.filebrowser.adapter.FilesAdapter;
import com.justep.filebrowser.common.IFileItemClickListener;
import com.justep.filebrowser.common.Utils;
import com.justep.filebrowser.global.FileAction;
import com.justep.filebrowser.global.FileBroadcastInfo;
import com.justep.filebrowser.global.FilePath;
import com.justep.filebrowser.model.FileInfoData;
import com.justep.filebrowser.model.FolderData;

import java.util.List;
import java.util.Locale;

public class FilterActivity extends AppCompatActivity {
  private final String tag = "FilterActivity";
  protected FilesAdapter mAdapter;
  protected List<FileInfoData> mFileInfoList;
  private ImageView mIvBack;
  private ViewGroup mEmptyFilesView;
  private TextView mTvInfo;
  private RecyclerView mRvFiles;
  private Activity mActivity;

  static int gSorttype = 0;

  private int mCurSortType = 0;
  private FolderData folder;

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_filter);

    mActivity = this;

    mIvBack = (ImageView)findViewById(R.id.iv_back);
    mIvBack.setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        onBackPressed();
      }
    });
    mTvInfo = (TextView)findViewById(R.id.tvHeader);

    Intent intent = getIntent();
    String sFiletype = intent.getStringExtra(FileBroadcastInfo.FILTER_INFO);
    mTvInfo.setText("\"企业文件中\"中的" + sFiletype + "文件");

    int iFileType = 0;
    if(sFiletype.equals("文档")){
      iFileType = FileInfoData.FileType_DOC;
    }else if(sFiletype.equals("图片")){
      iFileType = FileInfoData.FileType_PICTURE;
    }else if(sFiletype.equals("压缩包")){
      iFileType = FileInfoData.FileType_ZIP;
    }else if(sFiletype.equals("音乐")){
      iFileType = FileInfoData.FileType_MUSIC;
    }else if(sFiletype.equals("视频")){
      iFileType = FileInfoData.FileType_MOVIE;
    }

    mRvFiles = (RecyclerView)findViewById(R.id.folder_recylerview);
    mEmptyFilesView = (ViewGroup)findViewById(R.id.emptyFolderLayout);
    initViews(iFileType);
    updateView();
  }

  @Override
  public void onBackPressed() {
    super.onBackPressed();
    overridePendingTransition(R.anim.back_left_in, R.anim.back_right_out);
  }

  public void initViews(int filetype) {
    folder = new FolderData(FilePath.ROOT_DIR, filetype);
    mCurSortType = gSorttype;
    mFileInfoList = folder.getFolderInfos();
    mAdapter = new FilesAdapter(this, mFileInfoList);
    mRvFiles.setAdapter(mAdapter);
    mRvFiles.setLayoutManager(new LinearLayoutManager(this));
    mAdapter.setOnItemClickListener(new IFileItemClickListener() {
      @Override
      public void onClick(View v, final int position) {
        //openfile
        FileInfoData data = mFileInfoList.get(position);
        Utils.openFile(mActivity, data.path);
      }

      @Override
      public void onLongClick(int action, final int position) {
        //弹出菜单
        final FileInfoData data = mFileInfoList.get(position);
        switch (action) {
          case FileAction.RENAME:
          {
            int index = data.name.lastIndexOf('.');
            String oldname = data.name;
            String ext = "";
            if(index != -1) {
              ext = data.name.substring(index).toLowerCase(Locale.US);
              oldname = data.name.substring(0, index);
            }

            final String fileext = ext;
            Utils.inputDialog(mActivity, "请输入新名称", oldname, "重命名", new Utils.OnGetInputInfo() {
              public void getInputString(String info) {
                boolean ret = folder.renameFileByIndex(position, info + fileext);
                if(ret) {
                  notifyChangeData(position);
                } else {
                  Toast.makeText(mActivity, "重命名：" + info + "失败", Toast.LENGTH_SHORT).show();
                }
              }
            });
            }
            break;
          case FileAction.DELETE: {
            boolean ret = folder.deleteFileByIndex(position);
            if (ret) {
              mAdapter.notifyItemRemoved(position);
              updateView();
              Intent intent = new Intent(FileBroadcastInfo.DATA_CHANGE_ACTION);
              mActivity.sendBroadcast(intent);
            } else {
              Toast.makeText(mActivity.getApplicationContext(), "删除失败！！", Toast.LENGTH_SHORT).show();
            }
          }
          break;
        }
      }

      @Override
      public void onDeleteBtnCilck(View v, int position){
        //删除
        boolean ret = folder.deleteFileByIndex(position);
        if(ret){
          mAdapter.notifyItemRemoved(position);
          updateView();
          Intent intent = new Intent(FileBroadcastInfo.DATA_CHANGE_ACTION);
          mActivity.sendBroadcast(intent);
        }else{
          Toast.makeText(mActivity.getApplicationContext(), "删除失败！！", Toast.LENGTH_SHORT).show();
        }
      }

    });
  }

  public void notifyChangeData(int position){
    mAdapter.notifyItemChanged(position);
    Intent intent = new Intent(FileBroadcastInfo.DATA_CHANGE_ACTION);
    mActivity.sendBroadcast(intent);
  }

  public void updateView(){
    if(mFileInfoList != null && mFileInfoList.size() > 0) {
      mRvFiles.setVisibility(View.VISIBLE);
      mEmptyFilesView.setVisibility(View.GONE);
    }else{
      mRvFiles.setVisibility(View.GONE);
      mEmptyFilesView.setVisibility(View.VISIBLE);
    }
  }
}
