package com.justep.filebrowser;

/**
 * Created by lihong on 16-10-12.
 */

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageInfo;
import android.graphics.Color;
import android.support.v4.app.FragmentManager;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ListPopupWindow;
import android.widget.TextView;
import android.widget.Toast;

import com.justep.filebrowser.adapter.ListPopupWindowAdapter;
import com.justep.filebrowser.adapter.MyMenuItem;
import com.justep.filebrowser.common.IRefreshView;
import com.justep.filebrowser.common.Utils;
import com.justep.filebrowser.fragment.FragmentStackManager;
import com.justep.filebrowser.global.FileBroadcastInfo;
import com.justep.filebrowser.global.FilePath;
import com.justep.filebrowser.model.ShellUtils;
import com.justep.filebrowser.view.TitleBar;

import java.io.File;


public class FolderManagerActivity extends AppCompatActivity implements IRefreshView {
  static float mScale = 0.0f;
  private FragmentManager mFragmentManager;
  private FragmentStackManager mFragStackManager;
  private long mExitTime = 0;
  private Context mContext;

  private TitleBar mTitleBar;

  private ListPopupWindow mSortListPopupWindow;
  private ListPopupWindowAdapter mSortListPopupWindowAdapter;
  private MyMenuItem[] mSortArrayList = {
    new MyMenuItem("名称排序", R.mipmap.ic_sort_letter),
    new MyMenuItem("时间排序", R.mipmap.ic_sort_time)
  };
  private ViewGroup mLayoutTableHead;
  private TextView mTextViewSort;
  private Button mBtnPaster;


  private ViewGroup mLayoutTableSeletHead;
  private TextView mSelectTextViewSort;
  private TextView mTvSelectDelete;
  private boolean bSelectMode = false;

  private ListPopupWindow mFilterListPopupWindow;
  private ListPopupWindowAdapter mFilterListPopupWindowAdapter;

  private UpdateHeader mUpdateHeaderReceiver;

  private MyMenuItem[] mFilterArrayList = {new MyMenuItem("全部", R.mipmap.ic_text),
    new MyMenuItem("文档", R.mipmap.ic_word),
    new MyMenuItem("图片", R.mipmap.ic_picture),
    new MyMenuItem("压缩包", R.mipmap.ic_zip),
    new MyMenuItem("音乐", R.mipmap.ic_music),
    new MyMenuItem("视频", R.mipmap.ic_movie)};

  private ImageView mImageViewFilter;
  private ImageView mImageViewHome;

  public static final String COPY_CONFIRM_ACTION = "android.intent.action.home.copy.cancel";

  public static int dip2px(int dip) {
    return (int) (dip * mScale + 0.5f);
  }

  public static int px2dip(int px) {
    return (int) (px / mScale + 0.5f);
  }

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_filebrowser);

    mScale = this.getResources().getDisplayMetrics().density;

    mContext = this;

    boolean isImmersive = false;
    if (Utils.hasKitKat() && !Utils.hasLollipop()) {
      isImmersive = true;
      getWindow().addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
    } else if (Utils.hasLollipop()) {
      Window window = getWindow();
      window.clearFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS
        | WindowManager.LayoutParams.FLAG_TRANSLUCENT_NAVIGATION);
      window.getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
        | View.SYSTEM_UI_FLAG_LAYOUT_STABLE);
      window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
      isImmersive = true;
    }

    mTitleBar = (TitleBar) findViewById(R.id.defaultToolbar);

    mTitleBar.setImmersive(isImmersive);

    mTitleBar.setBackgroundColor(Color.parseColor("#ffffff"));

    mTitleBar.setLeftImageResource(R.mipmap.ic_back);
    mTitleBar.setLeftTextColor(Color.BLACK);
    mTitleBar.setLeftVisible(false);
    mTitleBar.setLeftClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        onBackPressed();
      }
    });

    mTitleBar.setTitleColor(Color.BLACK);
    mTitleBar.setSubTitleColor(Color.BLACK);
    mTitleBar.setDividerColor(Color.GRAY);

    mTitleBar.setActionTextColor(Color.parseColor("#2196F3"));

    mImageViewHome = (ImageView) mTitleBar.addAction(new com.justep.filebrowser.view.TitleBar.ImageAction(R.mipmap.ic_home) {
      @Override
      public void performAction(View view) {
        mFragStackManager.goHomeFragment();
      }
    });

    mImageViewHome.setVisibility(View.GONE);

    mImageViewFilter = (ImageView) mTitleBar.addAction(new com.justep.filebrowser.view.TitleBar.ImageAction(R.mipmap.ic_filter) {
      @Override
      public void performAction(View view) {
        //Toast.makeText(MainActivity.this, "点击了过滤", Toast.LENGTH_SHORT).show();
        //指定anchor
        mFilterListPopupWindow.setAnchorView(view);
        mFilterListPopupWindow.setVerticalOffset(-15);
        mFilterListPopupWindow.setHorizontalOffset(-60);
        mFilterListPopupWindow.setModal(true);
        mFilterListPopupWindow.show();
      }
    });

	  mImageViewFilter.setBackgroundResource(R.drawable.recycler_bg);

    ImageView mCollectView = (ImageView) mTitleBar.addAction(new com.justep.filebrowser.view.TitleBar.ImageAction(R.mipmap.ic_search) {
      @Override
      public void performAction(View view) {
//      Toast.makeText(MainActivity.this, "点击了搜索", Toast.LENGTH_SHORT).show();
        Intent intent = new Intent();
        intent.setClass(mContext, SearchActivity.class);
        startActivity(intent);
	      overridePendingTransition(R.anim.push_left_in, R.anim.push_left_out);
      }
    });

	  mCollectView.setBackgroundResource(R.drawable.recycler_bg);

	  View view = mTitleBar.addAction(new com.justep.filebrowser.view.TitleBar.TextAction("退出") {
      @Override
      public void performAction(View view) {
        //Toast.makeText(MainActivity.this, "点击了退出", Toast.LENGTH_SHORT).show();
        mFragStackManager.clear();
        finish();
      }
    });

	  view.setBackgroundResource(R.drawable.recycler_bg);

    createSortMenu();
    createFilterMenu();

    mLayoutTableHead = (ViewGroup) findViewById(R.id.header);
    mLayoutTableSeletHead = (ViewGroup) findViewById(R.id.selectheader);

    Button btnNew = (Button) findViewById(R.id.btnNew);
    btnNew.setOnClickListener(new View.OnClickListener() {
                                @Override
                                public void onClick(View v) {
                                  createFile();
                                }
                              });

    mTvSelectDelete = (TextView)findViewById(R.id.tvDelete);
    mTvSelectDelete.setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        mFragStackManager.commitDeleteItems();
      }
    });


    final Button btnSelectAll = (Button) findViewById(R.id.btnSelectAll);
    btnSelectAll.setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        if(btnSelectAll.getText().equals("全选")){
          btnSelectAll.setText("全不选");
          mFragStackManager.selectAllItem(true);
        }else{
          btnSelectAll.setText("全选");
          mFragStackManager.selectAllItem(false);
        }
      }
    });

    Button btnSelect = (Button) findViewById(R.id.btnSelected);
    btnSelect.setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        mLayoutTableSeletHead.setVisibility(View.VISIBLE);
        mLayoutTableHead.setVisibility(View.INVISIBLE);
        mFragStackManager.setSelectMode(true);
        btnSelectAll.setText("全选");
        bSelectMode = true;
        mTvSelectDelete.setVisibility(View.VISIBLE);
      }
    });

    Button btnSelectRet = (Button) findViewById(R.id.btnSelectRet);
    btnSelectRet.setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        mLayoutTableHead.setVisibility(View.VISIBLE);
        mLayoutTableSeletHead.setVisibility(View.INVISIBLE);
        mFragStackManager.setSelectMode(false);
        bSelectMode = false;
        mTvSelectDelete.setVisibility(View.GONE);
      }
    });

    mBtnPaster = (Button) findViewById(R.id.btnPaster);

    mBtnPaster.setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        if(mFragStackManager.commitPasterFile()) {
          mBtnPaster.setVisibility(View.GONE);
        }
      }
    });

    mUpdateHeaderReceiver = new UpdateHeader();
    IntentFilter filter = new IntentFilter();
    filter.addAction(FileBroadcastInfo.HEADER_CHANGE_ACTION);
    mContext.registerReceiver(mUpdateHeaderReceiver, filter);

    mFragmentManager = getSupportFragmentManager();
    mFragStackManager = FragmentStackManager.getInstance(mFragmentManager, this);
    mFragStackManager.switchFragment(this, FilePath.ROOT_DIR);
  }

  private void createSortMenu(){
    mSortListPopupWindow= new ListPopupWindow(this);

    //自定义Adapter
    mSortListPopupWindowAdapter = new ListPopupWindowAdapter(mSortArrayList, this);
    mSortListPopupWindow.setAdapter(mSortListPopupWindowAdapter);
    mSortListPopupWindow.setHeight(ViewGroup.LayoutParams.WRAP_CONTENT);

    mSortListPopupWindow.setWidth(ListPopupWindow.MATCH_PARENT);

    mSortListPopupWindow.setOnItemClickListener(new AdapterView.OnItemClickListener() {
      @Override
      public void onItemClick(AdapterView<?> arg0, View arg1, int position,long arg3) {
        MyMenuItem item = (MyMenuItem)mSortArrayList[position];
        mTextViewSort.setText(item.text);
        mSelectTextViewSort.setText(item.text);
        mSortListPopupWindow.dismiss();
        mFragStackManager.refreshSortType(position);
      }
    });


    ViewGroup layoutSort = (ViewGroup)findViewById(R.id.layoutSort);
    mTextViewSort = (TextView)findViewById(R.id.tvSort);
    layoutSort.setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        //指定anchor
        mSortListPopupWindow.setAnchorView(v);
        mSortListPopupWindow.setVerticalOffset(-2);
        mSortListPopupWindow.setHorizontalOffset(-50);
        mSortListPopupWindow.setModal(true);
        mSortListPopupWindow.show();
      }
    });

    layoutSort = (ViewGroup)findViewById(R.id.selectLayoutSort);
    mSelectTextViewSort = (TextView)findViewById(R.id.tvSelectSort);
    layoutSort.setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        //指定anchor
        mSortListPopupWindow.setAnchorView(v);
        mSortListPopupWindow.setVerticalOffset(-2);
        mSortListPopupWindow.setHorizontalOffset(-50);
        mSortListPopupWindow.setModal(true);
        mSortListPopupWindow.show();
      }
    });
  }

  private void createFilterMenu(){
    mFilterListPopupWindow= new ListPopupWindow(this);

    //自定义Adapter
    mFilterListPopupWindowAdapter = new ListPopupWindowAdapter(mFilterArrayList, this);
    mFilterListPopupWindow.setAdapter(mFilterListPopupWindowAdapter);
    mFilterListPopupWindow.setHeight(ViewGroup.LayoutParams.WRAP_CONTENT);

    mFilterListPopupWindow.setWidth(ListPopupWindow.MATCH_PARENT);

    mFilterListPopupWindow.setOnItemClickListener(new AdapterView.OnItemClickListener() {
      @Override
      public void onItemClick(AdapterView<?> arg0, View arg1, int position,long arg3) {
        MyMenuItem item = (MyMenuItem)mFilterArrayList[position];
        Intent intent = new Intent();
        intent.setClass(mContext, FilterActivity.class);
        intent.putExtra(FileBroadcastInfo.FILTER_INFO, item.text);
        startActivity(intent);
        overridePendingTransition(R.anim.push_left_in, R.anim.push_left_out);
        mFilterListPopupWindow.dismiss();
      }
    });
  }

  @Override
  public void onBackPressed() {
    if(bSelectMode){
      mLayoutTableHead.setVisibility(View.VISIBLE);
      mLayoutTableSeletHead.setVisibility(View.INVISIBLE);
      mFragStackManager.setSelectMode(false);
      bSelectMode = false;
      mTvSelectDelete.setVisibility(View.GONE);
      return;
    }

    String peek = mFragStackManager.getPeek();
    if(peek.equals(FilePath.ROOT_DIR)) {
      mFragStackManager.clear();
      finish();
    } else {
      mFragStackManager.removeTop();
    }
  }

  @Override
  protected void onDestroy() {
    mFragStackManager.clear();
    mContext.unregisterReceiver(mUpdateHeaderReceiver);
    super.onDestroy();
  }

  public void refreshInfo(String label){
    if(mImageViewHome == null){
      return;
    }

    if(!label.equals(FilePath.ROOT_DIR)){
      mImageViewHome.setVisibility(View.VISIBLE);
      int index = label.lastIndexOf('/') + 1;
      String title = label.substring(index);
      mTitleBar.setTitle(title);

      mTitleBar.setLeftVisible(true);
      if(index - 1 != FilePath.ROOT_DIR.length()) {
        String parentDir = label.substring(0, index - 1);
        index = parentDir.lastIndexOf('/') + 1;
        title = parentDir.substring(index);
        mTitleBar.setLeftText(title);
      }else{
        mTitleBar.setLeftText("根目录");
      }
    }else{
      mImageViewHome.setVisibility(View.GONE);
      mTitleBar.setTitle("");
      mTitleBar.setLeftVisible(false);
    }
  }

  private void createFile(){
    Utils.inputDialog(this, "创建文件夹", null, "创建", new Utils.OnGetInputInfo(){
      public void getInputString(String info){
        boolean ret = mFragStackManager.newFolder(info);
        if(!ret) {
          Toast.makeText(getApplicationContext(), "创建文件夹：" + info + "失败", Toast.LENGTH_SHORT).show();
        }
      }
    });
  }

  class UpdateHeader extends BroadcastReceiver {
    @Override
    public void onReceive(Context context, Intent intent) {
      String action = intent.getAction();
      if(action.equals(FileBroadcastInfo.HEADER_CHANGE_ACTION)){
        boolean bVisible = intent.getBooleanExtra(FileBroadcastInfo.SHOW_PASTER_INFO, false);
        if(bVisible){
          mBtnPaster.setVisibility(View.VISIBLE);
        }else{
          mBtnPaster.setVisibility(View.GONE);
        }
      }
    }
  }

}
