package com.justep.filebrowser;

/**
 * Created by lihong on 16-10-12.
 */

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.justep.filebrowser.adapter.FilesAdapter;
import com.justep.filebrowser.common.IFileItemClickListener;
import com.justep.filebrowser.common.Utils;
import com.justep.filebrowser.global.FileAction;
import com.justep.filebrowser.global.FileBroadcastInfo;
import com.justep.filebrowser.global.FilePath;
import com.justep.filebrowser.model.FileInfoData;
import com.justep.filebrowser.model.FolderData;

import java.util.List;

public class SearchActivity extends AppCompatActivity {
  private final String tag = "SearchActivity";
  protected FilesAdapter mAdapter;
  protected List<FileInfoData> mFileInfoList;
  private ImageView mIvBack;
  private ImageView mIvDelete;
  private EditText mEtSearch;
  private ViewGroup mEmptyFilesView;
  private TextView mTvInfo;
  private RecyclerView mRvFiles;
  private Activity mActivity;

  static int gSorttype = 0;

  private int mCurSortType = 0;
  private FolderData folder;

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_search);
    mActivity = this;

    mIvBack = (ImageView)findViewById(R.id.search_iv_back);
    mIvBack.setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        onBackPressed();
      }
    });

    mIvDelete = (ImageView)findViewById(R.id.search_iv_delete);
    mEtSearch = (EditText)findViewById(R.id.search_et_input);
    mIvDelete.setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
	      mEtSearch.setText("");
        mIvDelete.setVisibility(View.GONE);
      }
    });

    mEtSearch.addTextChangedListener(new TextWatcher() {

		  public void onTextChanged(CharSequence s, int start, int before, int count) {
			  // TODO Auto-generated method stub
        Log.v("tag", "onTextChanged");
		  }

		  public void beforeTextChanged(CharSequence s, int start, int count,
		                                int after) {
			  // TODO Auto-generated method stub
        Log.v("tag", "beforeTextChanged");
		  }

		  public void afterTextChanged(Editable s) {
        Log.v("tag", "afterTextChanged: " + s);
			  if(s.length() == 0) {
          mIvDelete.setVisibility(View.GONE);
          mRvFiles.setVisibility(View.GONE);
          mEmptyFilesView.setVisibility(View.VISIBLE);
          mTvInfo.setText("点击键盘上的\"搜索\"开始");
			  } else {
          mIvDelete.setVisibility(View.VISIBLE);
          String searchStr = s.toString();
          folder.searchFolderInfos(searchStr, mCurSortType);
          mFileInfoList = folder.getFolderInfos();
          if(mFileInfoList != null && mFileInfoList.size() > 0){
            mAdapter.setData(mFileInfoList);
            mRvFiles.setVisibility(View.VISIBLE);
            mEmptyFilesView.setVisibility(View.GONE);
          }else{
            mRvFiles.setVisibility(View.GONE);
            mEmptyFilesView.setVisibility(View.VISIBLE);
            mTvInfo.setText("无搜索结果");
          }
			  }
		  }
	  });

	  mEtSearch.setMaxLines(1);
    mEtSearch.setSingleLine(true);
    mEtSearch.setFocusable(true);
	  getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE | WindowManager.LayoutParams.FLAG_ALT_FOCUSABLE_IM);
	  getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_VISIBLE);

    mTvInfo = (TextView)findViewById(R.id.tvInfo);
    mTvInfo.setText("点击键盘上的\"搜索\"开始");
    mEmptyFilesView = (ViewGroup)findViewById(R.id.emptyFilesLayout);
    mRvFiles = (RecyclerView)findViewById(R.id.folder_recylerview);

    initViews();
  }

  public void initViews() {
    folder = new FolderData(FilePath.ROOT_DIR, true);
    mCurSortType = gSorttype;
    folder.updateFolderInfos(mCurSortType, false);
    mAdapter = new FilesAdapter(this, mFileInfoList);
    mRvFiles.setAdapter(mAdapter);
    mRvFiles.setLayoutManager(new LinearLayoutManager(this));
    mAdapter.setOnItemClickListener(new IFileItemClickListener() {
      @Override
      public void onClick(View v, final int position) {
        //openfile
        FileInfoData data = mFileInfoList.get(position);
        Utils.openFile(mActivity, data.path);
      }

      @Override
      public void onLongClick(int action, final int position) {
        //弹出菜单
        final FileInfoData data = mFileInfoList.get(position);
        switch (action) {
          case FileAction.RENAME:
            Utils.inputDialog(mActivity, "请输入新名称", data.name, "重命名", new Utils.OnGetInputInfo() {
              public void getInputString(String info) {
                boolean ret = folder.renameFileByIndex(position, info);
                if(ret) {
                  String searchStr = mEtSearch.getText().toString();
                  folder.searchFolderInfos(searchStr, mCurSortType);
                  mFileInfoList = folder.getFolderInfos();
                  if(mFileInfoList != null && mFileInfoList.size() > 0){
                    mAdapter.setData(mFileInfoList);
                    mRvFiles.setVisibility(View.VISIBLE);
                    mEmptyFilesView.setVisibility(View.GONE);
                  }else{
                    mRvFiles.setVisibility(View.GONE);
                    mEmptyFilesView.setVisibility(View.VISIBLE);
                    mTvInfo.setText("无搜索结果");
                  }
                  notifyChangeData(position);
                } else {
                  Toast.makeText(mActivity, "重命名：" + info + "失败", Toast.LENGTH_SHORT).show();
                }
              }
            });
            break;
          case FileAction.DELETE: {
            boolean ret = folder.deleteFileByIndex(position);
            if (ret) {
              notifyChangeData(position);
            } else {
              Toast.makeText(mActivity.getApplicationContext(), "删除失败！！", Toast.LENGTH_SHORT).show();
            }
          }
          break;
        }
      }

      @Override
      public void onDeleteBtnCilck(View v, int position){
        //删除
          boolean ret = folder.deleteFileByIndex(position);
          if(ret){
            mAdapter.notifyItemRemoved(position);
            Intent intent = new Intent(FileBroadcastInfo.DATA_CHANGE_ACTION);
            mActivity.sendBroadcast(intent);
          }else{
            Toast.makeText(mActivity.getApplicationContext(), "删除失败！！", Toast.LENGTH_SHORT).show();
          }
      }

    });
  }

	@Override
	public void onBackPressed() {
		super.onBackPressed();
		overridePendingTransition(R.anim.back_left_in, R.anim.back_right_out);
	}

  public void notifyChangeData(int position){
    mAdapter.notifyItemChanged(position);
    Intent intent = new Intent(FileBroadcastInfo.DATA_CHANGE_ACTION);
    mActivity.sendBroadcast(intent);
  }

}
