package com.justep.filebrowser;

/**
 * Created by lihong on 16-10-12.
 */


import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.graphics.Color;
import android.support.v4.app.FragmentManager;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.justep.filebrowser.common.IRefreshView;
import com.justep.filebrowser.common.ISelectInfo;
import com.justep.filebrowser.common.Utils;
import com.justep.filebrowser.fragment.SelectFragmentStackManager;
import com.justep.filebrowser.global.FilePath;
import com.justep.filebrowser.model.ShellUtils;
import com.justep.filebrowser.view.TitleBar;

import java.io.File;
import java.util.ArrayList;


public class SelectActivity extends AppCompatActivity  implements IRefreshView, ISelectInfo {

  private final String tag = "SelectActivity";
  private FragmentManager mFragmentManager;
  private SelectFragmentStackManager mSelectFragStackManager;
  protected ArrayList<String> mSelectFiles;
  private TitleBar mTitleBar;
  private ImageView mImageViewHome;
  private TextView mTvOk;
  private int mMaxSelected;
  private Context mContext;
  private String mExitCaption;

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_select);
    mMaxSelected = 9;
    mContext = this;

    Intent intent = getIntent();
    if(intent.hasExtra("maxSelectCount")){
      mMaxSelected = intent.getIntExtra("maxSelectCount", 9);
    }

    mExitCaption = "完成";
    if(intent.hasExtra("exitCaption")){
      mExitCaption = intent.getStringExtra("exitCaption");
    }

    mSelectFiles = new ArrayList<String>();

    mTitleBar = (TitleBar) findViewById(R.id.defaultToolbar);

    boolean isImmersive = false;
    if (Utils.hasKitKat() && !Utils.hasLollipop()) {
      isImmersive = true;
      getWindow().addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
    } else if (Utils.hasLollipop()) {
      Window window = getWindow();
      window.clearFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS
        | WindowManager.LayoutParams.FLAG_TRANSLUCENT_NAVIGATION);
      window.getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
        | View.SYSTEM_UI_FLAG_LAYOUT_STABLE);
      window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
      isImmersive = true;
    }

    mTitleBar.setImmersive(isImmersive);

    mTitleBar.setBackgroundColor(Color.parseColor("#ffffff"));

    mTitleBar.setLeftImageResource(R.mipmap.ic_back);
    mTitleBar.setLeftTextColor(Color.BLACK);
    mTitleBar.setLeftVisible(false);
    mTitleBar.setLeftClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        onBackPressed();
      }
    });

    mTitleBar.setTitleColor(Color.BLACK);
    mTitleBar.setSubTitleColor(Color.BLACK);
    mTitleBar.setDividerColor(Color.GRAY);

    mImageViewHome = (ImageView) mTitleBar.addAction(new com.justep.filebrowser.view.TitleBar.ImageAction(R.mipmap.ic_home) {
      @Override
      public void performAction(View view) {
        mSelectFragStackManager.goHomeFragment();
      }
    });

    mTitleBar.setActionTextColor(Color.parseColor("#2196F3"));
    View view = mTitleBar.addAction(new com.justep.filebrowser.view.TitleBar.TextAction("取消") {
      @Override
      public void performAction(View view) {
        mSelectFragStackManager.clear();
        mSelectFiles.clear();
        setResult(RESULT_CANCELED, null);
        finish();
      }
    });

    view.setBackgroundResource(R.drawable.recycler_bg);

    mTvOk = (TextView) findViewById(R.id.tvOk);
    mTvOk.setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        //Toast.makeText(mContext, mSelectFiles.toString(), Toast.LENGTH_SHORT).show();
        mSelectFragStackManager.clear();
        Intent intent = new Intent();
        intent.putExtra("selectedfiles", mSelectFiles);
        setResult(RESULT_OK, intent);
        finish();
        mSelectFiles.clear();
      }
    });

    mTvOk.setText(mExitCaption);
    mTvOk.setEnabled(false);


    mFragmentManager = getSupportFragmentManager();
    mSelectFragStackManager = SelectFragmentStackManager.getInstance(mFragmentManager, this, this);
    String path = Utils.getStoragePath(this, false);
    File testfile = new File(path);
    if(!testfile.exists()){
      Toast.makeText(this.getApplicationContext(), "路径不存在！！", Toast.LENGTH_SHORT).show();
    }

    try{
      PackageInfo pkg = getPackageManager().getPackageInfo(getApplication().getPackageName(), 0);
      String appName = pkg.applicationInfo.loadLabel(getPackageManager()).toString();
      FilePath.ROOT_DIR = path + "/" + appName;
    }catch (Exception e){
      e.printStackTrace();
      FilePath.ROOT_DIR = path + "/" + getPackageName();
    }


    File file = new File(FilePath.ROOT_DIR);
    if(!file.exists()){
      boolean ret = file.mkdir();
      if(!ret){
        Log.v("MainActivity", "创建文件夹失败！");
      }else{
        //ShellUtils.execCommand("cp -rf /sdcard/DCIM/QQPhoto/* " + FilePath.ROOT_DIR, false);
        Utils.copyAssert(mContext, FilePath.ROOT_DIR);
      }
    }
    mSelectFragStackManager.switchFragment(this, FilePath.ROOT_DIR);
  }

  @Override
  public void onBackPressed() {
    String peek = mSelectFragStackManager.getPeek();
    if (peek.equals(FilePath.ROOT_DIR)) {
      mSelectFragStackManager.clear();
      mSelectFiles.clear();
	    setResult(RESULT_CANCELED, null);
      finish();
    } else {
      mSelectFragStackManager.removeTop();
    }
  }

  public void refreshInfo(String label){
    if(mImageViewHome == null){
      return;
    }

    if(!label.equals(FilePath.ROOT_DIR)){
      mImageViewHome.setVisibility(View.VISIBLE);
      int index = label.lastIndexOf('/') + 1;
      String title = label.substring(index);
      mTitleBar.setTitle(title);

      mTitleBar.setLeftVisible(true);
      if(index - 1 != FilePath.ROOT_DIR.length()) {
        String parentDir = label.substring(0, index - 1);
        index = parentDir.lastIndexOf('/') + 1;
        title = parentDir.substring(index);
        mTitleBar.setLeftText(title);
      }else{
        mTitleBar.setLeftText("根目录");
      }
    }else{
      mImageViewHome.setVisibility(View.GONE);
      mTitleBar.setTitle("");
      mTitleBar.setLeftVisible(false);
    }
  }

  public boolean isSelected(String pathname){
    return mSelectFiles.contains(pathname);
  }

  public boolean isMaxSelected(){
    if(mSelectFiles.size() >= mMaxSelected){
      return true;
    }
    return false;
  }

  public void setSelectedState(String pathname, boolean bStatus){
    if(bStatus){
      mSelectFiles.add(pathname);
    }else{
      mSelectFiles.remove(pathname);
    }

    if(mSelectFiles.size() > 0) {
      mTvOk.setText(mExitCaption + "(" + mSelectFiles.size() + "/" + mMaxSelected + ")");
      mTvOk.setEnabled(true);
    }else{
      mTvOk.setText(mExitCaption);
      mTvOk.setEnabled(false);
    }
  }
}