package com.justep.filebrowser.adapter;

/**
 * Created by lihong on 16/10/10.
 */

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.ContextMenu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.justep.filebrowser.R;
import com.justep.filebrowser.common.IFileItemClickListener;
import com.justep.filebrowser.common.Utils;
import com.justep.filebrowser.global.FileAction;
import com.justep.filebrowser.model.FileInfoData;
import com.justep.filebrowser.view.SlidingButtonView;

import java.util.List;

public class FilesAdapter extends BaseAdapter<FilesAdapter.FileViewHolder>
	implements SlidingButtonView.IonSlidingButtonListener {

  private int mCurLongClickPosition;

  private SlidingButtonView mMenu = null;

  public FilesAdapter(Context context, List<FileInfoData> data) {
    super(context, data);
   }

  @Override
  public FileViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
		View view = mInflater.inflate(R.layout.list_item, parent, false);
		return new FileViewHolder(view);
  }

  protected void initEvent(final FileViewHolder holder, int position) {
    holder.mLayoutContent.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				//判断是否有删除菜单打开
				Log.i("FolderAdapter", "onClick");
				if (menuIsOpen()) {
					closeMenu();//关闭菜单
					return;
				}

				if(mFileItemClickListener != null) {
					int curposition = holder.getLayoutPosition();
					FileInfoData data = mData.get(curposition);
          mFileItemClickListener.onClick(v, curposition);
				}
			}
		});

    holder.mLayoutContent.setOnLongClickListener(new View.OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				Log.i("FolderAdapter", "onLongClick");
				//判断是否有删除菜单打开
				if (menuIsOpen()) {
					closeMenu();//关闭菜单
					return true;
				}

        mCurLongClickPosition = holder.getLayoutPosition();
				return false;
			}
		});

    holder.mBtnDelete.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				Log.i("FolderAdapter", "onDeleteClick");
				closeMenu();//关闭菜单

				if(mFileItemClickListener != null) {
					int curposition = holder.getLayoutPosition();
          mFileItemClickListener.onDeleteBtnCilck(v, curposition);
				}
			}
		});
	}

	public void setOnItemClickListener(IFileItemClickListener listener) {
		this.mFileItemClickListener = listener;
	}

	@Override
	public void onBindViewHolder(final FileViewHolder holder, int position) {
		setContentView(holder, holder.getLayoutPosition());
		holder.mLayoutContent.setBackgroundResource(R.drawable.recycler_bg);
		initEvent(holder, holder.getLayoutPosition());
		//设置内容布局的宽为屏幕宽度
		holder.mLayoutContent.getLayoutParams().width = Utils.getScreenWidth(mContext);
	}

  public void setData(List<FileInfoData> data){
    mData = data;
    notifyDataSetChanged();
  }

		@Override
  public int getItemCount() {
    if(mData == null){
      return 0;
    }
    return mData.size();
  }

  private final MenuItem.OnMenuItemClickListener onEditMenu = new MenuItem.OnMenuItemClickListener() {
    @Override
    public boolean onMenuItemClick(MenuItem item) {
      if(mFileItemClickListener != null) {
        mFileItemClickListener.onLongClick(item.getItemId(), mCurLongClickPosition);
      }
      return true;
    }
  };

  class FileViewHolder extends RecyclerView.ViewHolder implements View.OnCreateContextMenuListener {
    public SlidingButtonView mRlRoot;
    public TextView mTvName;
    public ImageView mIvIcon;
    public TextView mTvInfo;

    public TextView mBtnDelete;
    public View mLayoutContent;


    public FileViewHolder(View itemView) {
      super(itemView);
      mRlRoot = (SlidingButtonView)itemView.findViewById(R.id.list_item_root);
      mTvName = (TextView) itemView.findViewById(R.id.list_item_tv_name);
      mIvIcon = (ImageView) itemView.findViewById(R.id.list_item_icon);
      mTvInfo = (TextView) itemView.findViewById(R.id.list_item_tv_info);
      ImageView Chevron = (ImageView) itemView.findViewById(R.id.chevron);
      Chevron.setVisibility(View.GONE);

      mLayoutContent = (ViewGroup) itemView.findViewById(R.id.layout_content);
      mBtnDelete = (TextView) itemView.findViewById(R.id.tv_delete);
      mRlRoot.setSlidingButtonListener(FilesAdapter.this);
      itemView.setOnCreateContextMenuListener(this);
    }

		@Override
		public void onCreateContextMenu(ContextMenu menu, View v, ContextMenu.ContextMenuInfo menuInfo) {
      MenuItem Edit = menu.add(0, FileAction.RENAME, 0, "重命名");//groupId, itemId, order, title
      MenuItem Delete = menu.add(0, FileAction.DELETE, 0, "删除");

      Edit.setOnMenuItemClickListener(onEditMenu);
      Delete.setOnMenuItemClickListener(onEditMenu);
		}
  }

	protected void setContentView(FileViewHolder holder, int position) {
      FileInfoData data = mData.get(position);
      holder.mTvName.setText(data.name);
      holder.mTvInfo.setText(data.size + " " + data.strModDate);
      if (data.type == FileInfoData.FileType_DOC) {
        //holder.mIvIcon.setBackgroundResource(R.drawable.round_gray);
        holder.mIvIcon.setImageResource(R.mipmap.ic_word);
      } else if (data.type == FileInfoData.FileType_OTHER) {
        //holder.mIvIcon.setBackgroundResource(R.drawable.round_gray);
        holder.mIvIcon.setImageResource(R.mipmap.ic_default);
      } else if (data.type == FileInfoData.FileType_PICTURE) {
        //holder.mIvIcon.setBackgroundResource(R.drawable.round_gray);
        holder.mIvIcon.setImageResource(R.mipmap.ic_picture);
      } else if (data.type == FileInfoData.FileType_MUSIC) {
        //holder.mIvIcon.setBackgroundResource(R.drawable.round_gray);
        holder.mIvIcon.setImageResource(R.mipmap.ic_music);
      } else if (data.type == FileInfoData.FileType_MOVIE) {
        //holder.mIvIcon.setBackgroundResource(R.drawable.round_gray);
        holder.mIvIcon.setImageResource(R.mipmap.ic_movie);
      } else {
        //holder.mIvIcon.setBackgroundResource(R.drawable.round_gray);
        holder.mIvIcon.setImageResource(R.mipmap.ic_default);
      }
  }

  /**
   * 删除菜单打开信息接收
   */
  @Override
  public void onMenuIsOpen(View view) {
	  mMenu = (SlidingButtonView) view;
  }

  /**
   * 滑动或者点击了Item监听
   * @param slidingButtonView
   */
  @Override
  public void onDownOrMove(SlidingButtonView slidingButtonView) {
    if(menuIsOpen()){
      if(mMenu != slidingButtonView){
        closeMenu();
      }
    }
  }

  /**
   * 关闭菜单
   */
  public void closeMenu() {
    mMenu.closeMenu();
    mMenu = null;
  }
  /**
   * 判断是否有菜单打开
   */
  public Boolean menuIsOpen() {
    if(mMenu != null){
      return true;
    }
    //Log.i("FolderAdapter","mMenu为null");
    return false;
  }
}
