package com.justep.filebrowser.adapter;

/**
 * Created by lihong on 16/10/10.
 */

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.ContextMenu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.justep.filebrowser.R;
import com.justep.filebrowser.common.IFileItemClickListener;
import com.justep.filebrowser.common.Utils;
import com.justep.filebrowser.global.FileAction;
import com.justep.filebrowser.model.FileInfoData;
import com.justep.filebrowser.view.SlidingButtonView;

import java.util.ArrayList;
import java.util.List;

public class FolderAdapter extends BaseAdapter<FolderAdapter.FileViewHolder>
	implements SlidingButtonView.IonSlidingButtonListener {

  private int mCurLongClickPosition;

	//多选处理
  private boolean mSelectMode = false;
	private ArrayList<FileInfoData> mSelectedData;//记录选中的position
  private SlidingButtonView mMenu = null;

  public FolderAdapter(Context context, List<FileInfoData> data) {
    super(context, data);
    mSelectedData = new ArrayList<FileInfoData>();
  }

  @Override
  public FileViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
		View view = mInflater.inflate(R.layout.list_item, parent, false);
		return new FileViewHolder(view);
  }

  protected void initEvent(final FileViewHolder holder, final int position) {
		holder.mLayoutContent.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				//判断是否有删除菜单打开
				Log.i("FolderAdapter", "onClick");
				if (menuIsOpen()) {
					closeMenu();//关闭菜单
					return;
				}

				if(mFileItemClickListener != null) {
					int curposition = holder.getLayoutPosition();
					FileInfoData data = mData.get(curposition);
					if(mSelectMode) {
						switchSelectedState(curposition);
            return;
					}
          mFileItemClickListener.onClick(v, curposition);
				}
			}
		});

		holder.mLayoutContent.setOnLongClickListener(new View.OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				Log.i("FolderAdapter", "onLongClick");
				//判断是否有删除菜单打开
				if (menuIsOpen()) {
					closeMenu();//关闭菜单
					return true;
				}

        mCurLongClickPosition = holder.getLayoutPosition();
				return false;
			}
		});

		holder.mBtnDelete.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				Log.i("FolderAdapter", "onDeleteClick");
				closeMenu();//关闭菜单

				if(mFileItemClickListener != null) {
					int curposition = holder.getLayoutPosition();
          mFileItemClickListener.onDeleteBtnCilck(v, curposition);
				}
			}
		});
	}

	public void setOnItemClickListener(IFileItemClickListener listener) {
		this.mFileItemClickListener = listener;
	}

	@Override
	public void onBindViewHolder(final FileViewHolder holder, int position) {
		setContentView(holder, holder.getLayoutPosition());
		holder.mLayoutContent.setBackgroundResource(R.drawable.recycler_bg);
		initEvent(holder, holder.getLayoutPosition());
		//设置内容布局的宽为屏幕宽度
		holder.mLayoutContent.getLayoutParams().width = Utils.getScreenWidth(mContext);
	}

		@Override
  public int getItemCount() {
    return mData.size();
  }

  private final MenuItem.OnMenuItemClickListener onEditMenu = new MenuItem.OnMenuItemClickListener() {
    @Override
    public boolean onMenuItemClick(MenuItem item) {
      if(mFileItemClickListener != null) {
        mFileItemClickListener.onLongClick(item.getItemId(), mCurLongClickPosition);
      }
      return true;
    }
  };

  class FileViewHolder extends RecyclerView.ViewHolder implements View.OnCreateContextMenuListener {
      public SlidingButtonView mRlRoot;
      public TextView mTvName;
      public ImageView mIvIcon;
      public TextView mTvInfo;
      public ImageView mChevron;

      public TextView mBtnDelete;
      public View mLayoutContent;

      public FileViewHolder(View itemView) {
        super(itemView);
        mRlRoot = (SlidingButtonView)itemView.findViewById(R.id.list_item_root);
        mTvName = (TextView) itemView.findViewById(R.id.list_item_tv_name);
        mIvIcon = (ImageView) itemView.findViewById(R.id.list_item_icon);
        mTvInfo = (TextView) itemView.findViewById(R.id.list_item_tv_info);
        mChevron = (ImageView) itemView.findViewById(R.id.chevron);

        mLayoutContent = (ViewGroup) itemView.findViewById(R.id.layout_content);
        mBtnDelete = (TextView) itemView.findViewById(R.id.tv_delete);
				mRlRoot.setSlidingButtonListener(FolderAdapter.this);
				itemView.setOnCreateContextMenuListener(this);
      }

		  public RecyclerView.ViewHolder setChecked(boolean checked){
			  if(checked) {
				  mChevron.setVisibility(View.VISIBLE);
			  }else{
				  mChevron.setVisibility(View.INVISIBLE);
			  }
			  return this;
		  }

		@Override
		public void onCreateContextMenu(ContextMenu menu, View v,
																		ContextMenu.ContextMenuInfo menuInfo) {
			//menu.setHeaderTitle("Select The Action");
      MenuItem Edit = menu.add(0, FileAction.RENAME, 0, "重命名");//groupId, itemId, order, title
      MenuItem Copy = menu.add(0, FileAction.COPY, 0, "复制");
      MenuItem Cut = menu.add(0, FileAction.CUT, 0, "剪切");
      MenuItem Delete = menu.add(0, FileAction.DELETE, 0, "删除");

      Edit.setOnMenuItemClickListener(onEditMenu);
      Copy.setOnMenuItemClickListener(onEditMenu);
      Cut.setOnMenuItemClickListener(onEditMenu);
      Delete.setOnMenuItemClickListener(onEditMenu);
		}
  }

	protected void setContentView(FileViewHolder holder, int position) {
      FileInfoData data = mData.get(position);
      holder.mTvName.setText(data.name);
      if(data.isDir) {
        holder.mTvInfo.setText(data.subCount + "项  " + data.strModDate);
        //holder.mIvIcon.setBackgroundResource(R.drawable.round_blue);
        holder.mIvIcon.setImageResource(R.mipmap.ic_folder);
        if(!mSelectMode) {
          holder.mChevron.setImageResource(R.mipmap.ic_arrow);
          holder.mChevron.setVisibility(View.VISIBLE);
        }else{
          holder.mChevron.setImageResource(R.mipmap.ic_check);
          if(isSelected(position)) {
            holder.mChevron.setVisibility(View.VISIBLE);
          }else{
            holder.mChevron.setVisibility(View.INVISIBLE);
          }
        }
      }else {
        holder.mChevron.setImageResource(R.mipmap.ic_check);
        if (isSelected(position)) {
          holder.mChevron.setVisibility(View.VISIBLE);
        } else {
          holder.mChevron.setVisibility(View.INVISIBLE);
        }

        holder.mTvInfo.setText(data.size + " " + data.strModDate);
        if (data.type == FileInfoData.FileType_DOC) {
          //holder.mIvIcon.setBackgroundResource(R.drawable.round_gray);
          holder.mIvIcon.setImageResource(R.mipmap.ic_word);
        } else if (data.type == FileInfoData.FileType_OTHER) {
          //holder.mIvIcon.setBackgroundResource(R.drawable.round_gray);
          holder.mIvIcon.setImageResource(R.mipmap.ic_default);
        } else if (data.type == FileInfoData.FileType_PICTURE) {
          //holder.mIvIcon.setBackgroundResource(R.drawable.round_gray);
          holder.mIvIcon.setImageResource(R.mipmap.ic_picture);
        } else if (data.type == FileInfoData.FileType_MUSIC) {
          //holder.mIvIcon.setBackgroundResource(R.drawable.round_gray);
          holder.mIvIcon.setImageResource(R.mipmap.ic_music);
        } else if (data.type == FileInfoData.FileType_MOVIE) {
          //holder.mIvIcon.setBackgroundResource(R.drawable.round_gray);
          holder.mIvIcon.setImageResource(R.mipmap.ic_movie);
        } else {
          //holder.mIvIcon.setBackgroundResource(R.drawable.round_gray);
          holder.mIvIcon.setImageResource(R.mipmap.ic_default);
        }
      }
  }


  public void setSelectMode(boolean flag){
    mSelectMode = flag;
    if(menuIsOpen()) {
      closeMenu();
    }

    clearSelectedState();
    notifyDataSetChanged();
  }
	/**
	 * 判读是否选中
	 */
	public boolean isSelected(int position){
    FileInfoData data = mData.get(position);
    return mSelectedData.contains(data);
	}

	/**
	 * 切换选中或取消选中
	 */
	public void switchSelectedState(int position){
    if(!mSelectMode){
      return;
    }

    FileInfoData data = mData.get(position);
    if(mSelectedData.contains(data)){
      mSelectedData.remove(data);
    }else{
      mSelectedData.add(data);
    }
		notifyItemChanged(position);
	}

	/**
	 * 清除所有选中item的标记
	 */
	public void clearSelectedState(){
    mSelectedData.clear();
    notifyDataSetChanged();
	}

	/**
	 * 全选
	 */
	public void selectAllItems(){
    if(!mSelectMode){
      return;
    }

    mSelectedData.clear();
		for(int i = 0; i < getItemCount(); i++){
      mSelectedData.add(mData.get(i));
		}
    notifyDataSetChanged();
	}

	/**
	 * 获取item数目
	 */
	public int getSelectedItemCount(){
    if(!mSelectMode){
      mSelectedData.clear();
      return 0;
    }

		return mSelectedData.size();
	}

	/**
	 * 获取选中的items
	 */
	public List<FileInfoData> getSelectedItems(){
    if(!mSelectMode){
      return null;
    }
		return mSelectedData;
	}

  /**
   * 删除菜单打开信息接收
   */
  @Override
  public void onMenuIsOpen(View view) {
	  mMenu = (SlidingButtonView) view;
  }

  /**
   * 滑动或者点击了Item监听
   * @param slidingButtonView
   */
  @Override
  public void onDownOrMove(SlidingButtonView slidingButtonView) {
    if(menuIsOpen()){
      if(mMenu != slidingButtonView){
        closeMenu();
      }
    }
  }

  /**
   * 关闭菜单
   */
  public void closeMenu() {
    mMenu.closeMenu();
    mMenu = null;
  }
  /**
   * 判断是否有菜单打开
   */
  public Boolean menuIsOpen() {
    if(mMenu != null){
      return true;
    }
    //Log.i("FolderAdapter","mMenu为null");
    return false;
  }
}
