package com.justep.filebrowser.common;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.res.AssetManager;
import android.net.Uri;
import android.os.Build;
import android.os.storage.StorageManager;
import android.support.v4.app.ActivityCompat;
import android.support.v7.app.AlertDialog;
import android.util.DisplayMetrics;
import android.view.WindowManager;
import android.webkit.MimeTypeMap;
import android.widget.EditText;
import android.widget.Toast;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.lang.reflect.Array;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Locale;

/**
 * Created by lihong on 2016/10/20.
 */

public class Utils {
  public static int getScreenWidth(Context context)
  {
    WindowManager wm = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
    DisplayMetrics outMetrics = new DisplayMetrics();
    wm.getDefaultDisplay().getMetrics( outMetrics);
    return outMetrics .widthPixels ;
  }

  public static void openFile(Activity activity, String filePath)
  {
    try
    {
      String mime = "text/plain";
      int index = filePath.lastIndexOf('.');
      if(index != -1){
        String ext = filePath.substring(index + 1).toLowerCase(Locale.US);
        MimeTypeMap mimeTypeMap = MimeTypeMap.getSingleton();
        String tmpmime = mimeTypeMap.getMimeTypeFromExtension(ext);
        if(tmpmime != null){
          mime = tmpmime;
        }
      }

      Intent intent = new Intent("android.intent.action.VIEW");
      intent.addCategory("android.intent.category.DEFAULT");
      intent.setAction(android.content.Intent.ACTION_VIEW);
      intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

      File file = new File(filePath);
      intent.setDataAndType(Uri.fromFile(file), mime);
      activity.startActivity(intent);
    }catch (Exception e) {
      e.printStackTrace();
      Toast.makeText(activity, "无法打开该文件", Toast.LENGTH_SHORT).show();
    }
  }

  public interface OnGetInputInfo{
    public void getInputString(String info);
  }

  public static void inputDialog(final Context context, String title, String inittxt, String okBtnInfo, final OnGetInputInfo getInputInfo) {
    final EditText inputServer = new EditText(context);
    inputServer.setMaxLines(1);
    inputServer.setSingleLine(true);
    inputServer.setFocusable(true);
    if(inittxt != null){
      inputServer.setText(inittxt);
    }

    AlertDialog.Builder builder = new AlertDialog.Builder(context);
    builder.setTitle(title).setView(inputServer).setNegativeButton(
      "取消", null);
    builder.setPositiveButton(okBtnInfo,
      new DialogInterface.OnClickListener() {
        public void onClick(DialogInterface dialog, int which) {
          String info = inputServer.getText().toString();
          if (info == null || info.length() == 0) {
            Toast.makeText(context.getApplicationContext(), "输入不能为空", Toast.LENGTH_SHORT).show();
            return;
          }
          getInputInfo.getInputString(info);
        }
      });
    AlertDialog alertdialog = builder.create();
    alertdialog.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE | WindowManager.LayoutParams.FLAG_ALT_FOCUSABLE_IM);
    alertdialog.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_VISIBLE);
    alertdialog.show();
  }

  public static String getStoragePath(Context mContext, boolean is_removale) {

    StorageManager mStorageManager = (StorageManager) mContext.getSystemService(Context.STORAGE_SERVICE);
    Class<?> storageVolumeClazz = null;
    try {
      storageVolumeClazz = Class.forName("android.os.storage.StorageVolume");
      Method getVolumeList = mStorageManager.getClass().getMethod("getVolumeList");
      Method getPath = storageVolumeClazz.getMethod("getPath");
      Method isRemovable = storageVolumeClazz.getMethod("isRemovable");
      Object result = getVolumeList.invoke(mStorageManager);
      final int length = Array.getLength(result);
      for (int i = 0; i < length; i++) {
        Object storageVolumeElement = Array.get(result, i);
        String path = (String) getPath.invoke(storageVolumeElement);
        boolean removable = (Boolean) isRemovable.invoke(storageVolumeElement);
        if (is_removale == removable) {
          return path;
        }
      }
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
    } catch (InvocationTargetException e) {
      e.printStackTrace();
    } catch (NoSuchMethodException e) {
      e.printStackTrace();
    } catch (IllegalAccessException e) {
      e.printStackTrace();
    }
    return null;
  }

  public static boolean hasKitKat() {
    return Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT;
  }

  public static boolean hasLollipop() {
    return Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP;
  }

  public static void copyAssert(Context context, String savePath) {
    File dir = new File(savePath);
    if(!dir.exists()) {
      dir.mkdir();
    }

    AssetManager assets = context.getAssets();
    try {
      String[] fileNames = assets.list("");
      if(fileNames.length == 0){
        return;
      }

      for(String name : fileNames) {
        String[] childNames = assets.list(name);
        if(childNames.length > 0){
          continue;
        }

        String filename = savePath + "/" + name;
        File file = new File(filename);
        if(file.exists()){
          continue;
        }

        InputStream is = context.getResources().getAssets().open(name);
        FileOutputStream fos = new FileOutputStream(filename);
        byte[] buffer = new byte[4096];
        int count = 0;
        while ((count = is.read(buffer)) > 0) {
          fos.write(buffer, 0, count);
        }
        fos.close();
        is.close();
      }
    }catch (Exception e) {
      e.printStackTrace();
    }
  }

  private static final int REQUEST_EXTERNAL_STORAGE = 1;
  private static String[] PERMISSIONS_STORAGE = {
    Manifest.permission.READ_EXTERNAL_STORAGE,
    Manifest.permission.WRITE_EXTERNAL_STORAGE };

  public static void verifyStoragePermissions(Activity activity) {
//    int permission = ActivityCompat.checkSelfPermission(activity,
//      Manifest.permission.WRITE_EXTERNAL_STORAGE);
//
//    if (permission != PackageManager.PERMISSION_GRANTED) {
//      ActivityCompat.requestPermissions(activity, PERMISSIONS_STORAGE,
//        REQUEST_EXTERNAL_STORAGE);
//    }
  }
}
