package com.justep.filebrowser.fragment;

/**
 * Created by lihong on 16-10-12.
 */

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.view.View;
import android.widget.Toast;

import com.justep.filebrowser.adapter.BaseAdapter;
import com.justep.filebrowser.adapter.FolderAdapter;
import com.justep.filebrowser.common.Utils;
import com.justep.filebrowser.global.FileAction;
import com.justep.filebrowser.global.FileBroadcastInfo;
import com.justep.filebrowser.model.FileInfoData;
import com.justep.filebrowser.model.FolderData;


import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

public class DirFragment extends ListFragment {
  static int gSorttype = 0;

  private int mCurSortType = 0;

  private String mCurPath;
  private FolderData mFolder;
  RefreshData mRefreshData;

  public DirFragment(Activity activity, String mLabel) {
    super(mLabel);
    this.mCurPath = mLabel;
    mFolder = new FolderData(mLabel, false);
    mCurSortType = gSorttype;
    mFolder.updateFolderInfos(mCurSortType, false);

    mRefreshData = new DirFragment.RefreshData();
    IntentFilter filter = new IntentFilter();
    filter.addAction(FileBroadcastInfo.DATA_CHANGE_ACTION);
    activity.registerReceiver(mRefreshData, filter);
  }


  @Override
  protected BaseAdapter newAdapter(Activity activity, List<FileInfoData> infoList) {
      return new FolderAdapter(activity, infoList);
  }

  @Override
  protected void onItemClick(View v, int position) {
    FileInfoData data = mFileInfoList.get(position);
    if(data == null){
      return;
    }

    File file = new File(data.path);
    if(file.isDirectory()) {
      FragmentStackManager.getInstance().switchFragment(mActivity, data.path);
      return;
    }
    Utils.openFile(mActivity, data.path);
  }

  private  String nameGenerator(String oldname) {
    String ctime = new SimpleDateFormat("HH:mm:ss").format(new Date());
    String[] token = oldname.split(".");
    if(token.length < 2){
      return oldname + "-" + ctime;
    }
    return token[0] + "-" + ctime + "." + token[1];
  }

  @Override
  protected void onItemLongClick(int action, final int position) {
    final FileInfoData data = mFileInfoList.get(position);
    switch (action){
      case FileAction.RENAME:
        Utils.inputDialog(mActivity, "请输入新名称", data.name, "重命名", new Utils.OnGetInputInfo(){
          public void getInputString(String info){
            boolean ret = mFolder.renameFile(data.name, info);
            if(!ret) {
              Toast.makeText(mActivity, "重命名：" + info + "失败", Toast.LENGTH_SHORT).show();
            }else{
              mAdapter.notifyItemChanged(position);
              refreshData();
            }
          }
        });
        break;
      case FileAction.COPY:
        {
          String newName = nameGenerator(data.name);
          boolean ret = mFolder.copyFile(data.name, newName);
          if(ret){
            refreshData();
          }else{
            Toast.makeText(mActivity, "复制：" + data.name + "失败", Toast.LENGTH_SHORT).show();
          }
        }
        break;
      case FileAction.CUT:
        mFolder.setOrgPathName(data.path, 1);
        break;
      case FileAction.DELETE:
        {
          boolean ret = mFolder.deleteFileByIndex(position);
          if (ret) {
            mAdapter.notifyItemRemoved(position);
            refreshData();
          } else {
            Toast.makeText(mActivity.getApplicationContext(), "删除失败！！", Toast.LENGTH_SHORT).show();
          }
        }
        break;
    }
  }

  @Override
  protected void onItemDeleteCilck(View view, int position){
    boolean ret = mFolder.deleteFileByIndex(position);
    if(ret){
      mAdapter.notifyItemRemoved(position);
      refreshData();
    }else{
      Toast.makeText(mActivity.getApplicationContext(), "删除失败！！", Toast.LENGTH_SHORT).show();
    }
  }

  @Override
  protected List<FileInfoData> getData() {
    List<FileInfoData> list = mFolder.getFolderInfos();
    return list;
  }

  public String getPath() {
      return mCurPath;
  }

  private void refreshData() {
    Intent intent = new Intent(FileBroadcastInfo.DATA_CHANGE_ACTION);
    mActivity.sendBroadcast(intent);
  }

  @Override
  public void initData() {
    super.initData();
    refreshPasterStatus();
  }

  @Override
  public void onDestroy() {
    super.onDestroy();
    mActivity.unregisterReceiver(mRefreshData);
    mFolder.clear();
  }

//  @Override
//  public boolean onContextItemSelected(MenuItem item) {
//    Log.v("DirFragment", "item id:" + item.getItemId());
//    return true;
//  }

  public boolean newFolder(String foldername) {
    if(mFolder != null){
      boolean ret = mFolder.createDirectoryAtPath(foldername);
      if(ret){
        refreshData();
      }
      return ret;
    }
    return  false;
  }

  public void sortFiles() {
    if(mCurSortType == gSorttype){
      return;
    }

    mCurSortType = gSorttype;
    mFolder.updateFolderInfos(gSorttype, false);
    mAdapter.notifyDataSetChanged();
  }

  public void setSelectMode(boolean flag){
    ((FolderAdapter)mAdapter).setSelectMode(flag);
  }

  public void selectAllItem(boolean flag){
    if(flag){
      ((FolderAdapter)mAdapter).selectAllItems();
    }else{
      ((FolderAdapter)mAdapter).clearSelectedState();
    }
  }

  public void deleteSelectdItems(){
    List<FileInfoData> items = ((FolderAdapter)mAdapter).getSelectedItems();
    for(FileInfoData data: items){
      mFolder.deleteFile(data.name);
    }
    ((FolderAdapter)mAdapter).clearSelectedState();
    refreshData();
  }

  public boolean pasterFile(){
    int ret = mFolder.pasteFile();
    if(ret == 0){
      return true;
    }

    if(ret == 1){
      Toast.makeText(mActivity.getApplicationContext(), "不能移动到同路径的子文件夹中！", Toast.LENGTH_SHORT).show();
      return false;
    }

    Toast.makeText(mActivity.getApplicationContext(), "粘贴失败！", Toast.LENGTH_SHORT).show();
    return false;
  }

  public void refreshPasterStatus(){
    boolean status = mFolder.getPasteStatus();
    Intent intent = new Intent(FileBroadcastInfo.HEADER_CHANGE_ACTION);
    intent.putExtra(FileBroadcastInfo.SHOW_PASTER_INFO, status);
    mActivity.sendBroadcast(intent);
  }

  class RefreshData extends BroadcastReceiver {
    @Override
    public void onReceive(Context context, Intent intent) {
      String action = intent.getAction();
      if(action.equals(FileBroadcastInfo.DATA_CHANGE_ACTION)){
        if(mRvRootDir != null) {
          mFileInfoList.clear();
          mFolder.updateFolderInfos(mCurSortType, true);
          mAdapter.notifyDataSetChanged();
          updateView();
        }
      }
    }
  }


}
