package com.justep.filebrowser.fragment;

/**
 * Created by lihong on 16/10/10.
 */

import android.app.Activity;
import android.content.Intent;
import android.support.v4.app.FragmentTransaction;

import com.justep.filebrowser.R;
import com.justep.filebrowser.common.IRefreshView;
import com.justep.filebrowser.global.FileBroadcastInfo;
import com.justep.filebrowser.global.FilePath;



public class FragmentStackManager {
    private static FragmentStackManager sManager;
    private FragmentStack mFragmentStack;
    private android.support.v4.app.FragmentManager mFragmentManager;
    private IRefreshView iRefreshView;
	  private int mRootIndex;

    private FragmentStackManager(android.support.v4.app.FragmentManager fm, IRefreshView iRefresh) {
        mFragmentStack = new FragmentStack();
        this.mFragmentManager = fm;
        this.iRefreshView = iRefresh;
    }

    private FragmentStackManager(android.support.v4.app.FragmentManager fm) {
        mFragmentStack = new FragmentStack();
        this.mFragmentManager = fm;
    }

    private FragmentStackManager() {
        mFragmentStack = new FragmentStack();
    }

    public synchronized static FragmentStackManager getInstance(android.support.v4.app.FragmentManager fm, IRefreshView iRefreshView) {
        if (sManager == null) {
            sManager = new FragmentStackManager(fm, iRefreshView);
        }
        return sManager;
    }


    public synchronized static FragmentStackManager getInstance(android.support.v4.app.FragmentManager fm) {
        if (sManager == null) {
            sManager = new FragmentStackManager(fm);
        }
        return sManager;
    }

    public synchronized static FragmentStackManager getInstance() {
        if (sManager == null) {
            sManager = new FragmentStackManager();
        }
        return sManager;
    }

    public void switchFragment(Activity activity, String label) {
        BaseFragment fragment = null;
        FragmentTransaction ft = mFragmentManager.beginTransaction();
        //如果栈中不存在fragment，则新建一个并且push
        if(!mFragmentStack.hasFragment(label)) {
            fragment = new DirFragment(activity, label);
            mFragmentStack.push(fragment);
            if(!label.equals(FilePath.ROOT_DIR)){
                ft.setCustomAnimations(
                  R.anim.push_left_in,
	                R.anim.push_left_out,
                  R.anim.back_left_in,
                  R.anim.back_right_out);
	            ft.replace(R.id.forldView, fragment, label);
	            ft.addToBackStack(null);
	            ft.commit();
            }else {
	            ft.replace(R.id.forldView, fragment, label);
	            ft.addToBackStack(null);
	            mRootIndex = ft.commit();
            }
        }
        mFragmentStack.printData();
        label = fragment.getLabel();
        iRefreshView.refreshInfo(label);
    }

    public void goHomeFragment() {
      BaseFragment fragment = null;
      String path = getPeek();
      while(!path.equals(FilePath.ROOT_DIR)){
        mFragmentStack.pop();
        path = getPeek();
      }

      fragment = mFragmentStack.peek();
      mFragmentStack.clear();
      mFragmentStack.push(fragment);
      mFragmentStack.printData();
      if(fragment != null && fragment instanceof DirFragment){
        ((DirFragment) fragment).sortFiles();
        ((DirFragment) fragment).refreshPasterStatus();
//        if(needRefreshData) {
//          ((DirFragment) fragment).refreshData();
//          needRefreshData = false;
//        }
      }

	    mFragmentManager.popBackStack(mRootIndex, 0);
      iRefreshView.refreshInfo(FilePath.ROOT_DIR);
    }

    public void clear() {
        mFragmentStack.clear();
        sManager = null;
    }

    public String getPeek() {
        return mFragmentStack.peek().getLabel();
    }

    public void removeTop() {
        mFragmentStack.pop();
        BaseFragment fragment = mFragmentStack.peek();
        if(fragment != null && fragment instanceof DirFragment){
          ((DirFragment) fragment).sortFiles();
          ((DirFragment) fragment).refreshPasterStatus();
        }

        iRefreshView.refreshInfo(fragment.getLabel());
		    int count = mFragmentManager.getBackStackEntryCount();
		    if(count > 0) {
			    mFragmentManager.popBackStack();
		    }
    }

    public boolean newFolder(String name){
      if(name != null && name.length() > 0){
        BaseFragment fragment = mFragmentStack.peek();
        if(fragment != null && fragment instanceof DirFragment){
          return ((DirFragment) fragment).newFolder(name);
        }
      }
      return false;
    }

    public void refreshSortType(int sortType){
      if(DirFragment.gSorttype != sortType){
        DirFragment.gSorttype = sortType;
        BaseFragment fragment = mFragmentStack.peek();
        if(fragment != null && fragment instanceof DirFragment){
          ((DirFragment) fragment).sortFiles();
        }
      }
    }

    public void setSelectMode(boolean flag){
      BaseFragment fragment = mFragmentStack.peek();
      if(fragment != null && fragment instanceof DirFragment){
        ((DirFragment) fragment).setSelectMode(flag);
      }
    }

    public void selectAllItem(boolean flag){
      BaseFragment fragment = mFragmentStack.peek();
      if(fragment != null && fragment instanceof DirFragment){
        ((DirFragment) fragment).selectAllItem(flag);
      }
    }

    public void commitDeleteItems(){
      BaseFragment fragment = mFragmentStack.peek();
      if(fragment != null && fragment instanceof DirFragment){
        ((DirFragment) fragment).deleteSelectdItems();
        Intent intent = new Intent(FileBroadcastInfo.DATA_CHANGE_ACTION);
        fragment.mActivity.sendBroadcast(intent);
      }
    }

    public boolean commitPasterFile(){
      BaseFragment fragment = mFragmentStack.peek();
      if(fragment != null && fragment instanceof DirFragment){
        boolean ret = ((DirFragment) fragment).pasterFile();
        if(ret){
          Intent intent = new Intent(FileBroadcastInfo.DATA_CHANGE_ACTION);
          fragment.mActivity.sendBroadcast(intent);
        }
        return ret;
      }
      return false;
    }
}
