//
//  FolderData.h
//  FileBrowser
//
//  Created by leeh on 16/9/7.
//  Copyright © 2016年 leeh. All rights reserved.
//
package com.justep.filebrowser.model;


/**
 * Created by lihong on 16/10/11.
 */

import java.io.File;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import com.justep.filebrowser.model.ShellUtils;

public class  FolderData {
  File folder;
  String curPath;
  ArrayList<FileInfoData> contents;
  boolean isSearch;
  ArrayList<String> files;

  static String gOrgPathname = null;
  static int gOperateType = -1; //0为拷贝，1为剪切

  public List<FileInfoData> getFolderInfos(){
    return contents;
  }

  public FolderData(String path, boolean bSearch){
    if(path == null || path.length() == 0) {
      return;
    }

    File file = new File(path);
    if(file.exists() == false) {
      return;
    }

    this.folder = file;
    this.isSearch = bSearch;
    this.curPath = path + "/";
    if(isSearch){
      enumFileName(file);
    }
  }

  private void enumFileName(File file){
    if(file.exists() == false) {
      return;
    }

    if(file.isFile()) {
      if(files == null){
        files = new ArrayList<String>();
      }

      this.files.add(file.getPath());
      return;
    }

    //如果是目录则递归计算其内容的总大小
    if(file.isDirectory() && !file.isHidden()) {
      File[] children = file.listFiles();
      if (children == null) {
        return;
      }

      for (File f : children) {
        if (f.exists()) {
          enumFileName(f);
        }
      }
    }
  }

  public FolderData(String path, int filetype) {
    if(path == null || path.length() == 0) {
      return;
    }

    File file = new File(path);
    if(file.exists() == false) {
      return;
    }

    this.folder = file;
    this.isSearch = false;
    this.curPath = path + "/";
    if(contents == null){
      contents = new ArrayList<FileInfoData>();
    }
    enumFolderInfoData(file, filetype);
  }

  private void enumFolderInfoData(File file, int filetype){
    if(file.exists() == false) {
      return;
    }

    if(file.isFile() && !file.isHidden()){

      FileInfoData data = getFileInfoObject(file);
      if(filetype == FileInfoData.FileType_PICTURANDEMOVIE &&
        (data.type == FileInfoData.FileType_PICTURE || data.type == FileInfoData.FileType_MOVIE)) {
        this.contents.add(data);
      }else if(data.type == filetype || filetype == 0){
        this.contents.add(data);
      }
      return;
    }

    //如果是目录则递归计算其内容的总大小
    if(file.isDirectory() && !file.isHidden()) {
      File[] children = file.listFiles();
      if (children == null) {
        return;
      }

      for (File f : children) {
        if (f.exists()) {
          enumFolderInfoData(f, filetype);
        }
      }
    }
  }

  private FileInfoData getFileInfoObject(File file){
    FileInfoData infodata = new FileInfoData();
    if(file.isDirectory()) {
      infodata.isDir = true;
      infodata.name = file.getName();
      if(file.list() != null) {
        infodata.subCount = file.list().length;
      }
    }else {
      infodata.isDir = false;
      infodata.name = file.getName();
      infodata.subCount = 0;
      infodata.type = getType(infodata.name);
      infodata.nSize = file.length();
      infodata.size = getSize(infodata.nSize);
    }

    infodata.path = file.getPath();
    long time = file.lastModified();
    String ctime = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss").format(new Date(time));
    infodata.strModDate = ctime;
    infodata.modDate = new Date(time);
    return infodata;
  }

  private static String getSize(long length) {
    if (length < 1024)
      return length + "B";
    else if (1024 <= length && length < Math.pow(1024, 2)) {
      float scale = (float) length / 1024;
      DecimalFormat fnum = new DecimalFormat("##0.00");
      String dd = fnum.format(scale) + "KB";
      return dd;
    } else if (Math.pow(1024, 2) <= length && length < Math.pow(1024, 3)) {
      float scale = (float) length / 1024 / 1024;
      DecimalFormat fnum = new DecimalFormat("##0.00");
      String dd = fnum.format(scale) + "MB";
      return dd;
    } else {
      float scale = (float) length / 1024 / 1024 / 1024;
      DecimalFormat fnum = new DecimalFormat("##0.00");
      String dd = fnum.format(scale) + "GB";
      return dd;
    }
  }

  public static int getType(String name) {
    String[] strs = name.split("\\.");
    if(strs.length == 0) {
      return FileInfoData.FileType_OTHER;
    }

    String postfix = strs[strs.length - 1];
    if(postfix.equalsIgnoreCase("txt") ||
      postfix.equalsIgnoreCase("doc") ||
      postfix.equalsIgnoreCase("c") ||
      postfix.equalsIgnoreCase("java") ||
      postfix.equalsIgnoreCase("html") ||
      postfix.equalsIgnoreCase("rc") ||
      postfix.equalsIgnoreCase("py") ||
      postfix.equalsIgnoreCase("ini") ||
      postfix.equalsIgnoreCase("rtf") ||
      postfix.equalsIgnoreCase("docx") ||
      postfix.equalsIgnoreCase("els")) {
      return FileInfoData.FileType_DOC;
    } else if (postfix.equalsIgnoreCase("mp3") ||
      postfix.equalsIgnoreCase("wav") ||
      postfix.equals("flac")) {
      return FileInfoData.FileType_MUSIC;
    } else if (postfix.equalsIgnoreCase("mp4") ||
      postfix.equalsIgnoreCase("avi") ||
      postfix.equalsIgnoreCase("3gp") ||
      postfix.equalsIgnoreCase("wmv")) {
      return FileInfoData.FileType_MOVIE;
    } else if (postfix.equalsIgnoreCase("zip") ||
      postfix.equalsIgnoreCase("gzip") ||
      postfix.equalsIgnoreCase("rar") ||
      postfix.equalsIgnoreCase("tar")) {
      return FileInfoData.FileType_ZIP;
    }else if (postfix.equalsIgnoreCase("jpg") ||
      postfix.equalsIgnoreCase("png") || postfix.equalsIgnoreCase("jpeg") ||
      postfix.equalsIgnoreCase("bmp") || postfix.equalsIgnoreCase("gif")) {
      return FileInfoData.FileType_PICTURE;
    } else if(postfix.equalsIgnoreCase("pdf")) {
      return FileInfoData.FileType_PDF;
    }else {
      return FileInfoData.FileType_OTHER;
    }
  }

  public void updateFolderInfos(int sorttype, boolean needreload){
    if(needreload || contents == null) {
      if (isSearch) {
        return;
      }

      if (contents == null) {
        contents = new ArrayList<>();
      } else {
        contents.clear();
      }

      File file = new File(curPath);
      if (!file.exists()) {
        return;
      }

      File[] children = file.listFiles();
      if (children == null) {
        return;
      }

      for (File f : children) {
        FileInfoData data = getFileInfoObject(f);
        contents.add(data);
      }
    }

    //名称排序
    if(sorttype == 0){
      Collections.sort(contents, new Comparator<FileInfoData>() {
        @Override
        public int compare(FileInfoData lhs, FileInfoData rhs) {
          if(lhs.isDir && !rhs.isDir){
            return 1;
          }

          if(!lhs.isDir && rhs.isDir){
            return -1;
          }
          return lhs.name.compareTo(rhs.name);
        }
      });
      return;
    }

    //时间排序
    Collections.sort(contents, new Comparator<FileInfoData>() {
      @Override
      public int compare(FileInfoData lhs, FileInfoData rhs) {
        if(lhs.isDir && !rhs.isDir){
          return 1;
        }

        if(!lhs.isDir && rhs.isDir){
          return -1;
        }
        return lhs.modDate.compareTo(rhs.modDate);
      }
    });
  }

  public void searchFolderInfos(String searchstr,  int sorttype){
    ArrayList<FileInfoData> tmpfileinfos = new ArrayList<>();
    int i = 0;
    if(files == null || files.size() == 0){
      return;
    }

    if(searchstr != null || searchstr.length() > 0){
      for(String pathname:files) {
        String fname = pathname.substring(pathname.lastIndexOf("/") + 1).toLowerCase();
        if (fname.contains(searchstr.toLowerCase())) {
          File file = new File(pathname);
          FileInfoData data = getFileInfoObject(file);
          data.tag = i;
          tmpfileinfos.add(data);
        }
        i++;
      }
    }else{
      for(String pathname:files) {
        File file = new File(pathname);
        FileInfoData data = getFileInfoObject(file);
        data.tag = i++;
        tmpfileinfos.add(data);
      }
    }

    //名称排序
    if(sorttype == 0){
      Collections.sort(tmpfileinfos, new Comparator<FileInfoData>() {
        @Override
        public int compare(FileInfoData lhs, FileInfoData rhs) {
          if(lhs.isDir && !rhs.isDir){
            return 1;
          }

          if(!lhs.isDir && rhs.isDir){
            return -1;
          }

          return lhs.path.compareTo(rhs.path);
        }
      });
    }

    //时间排序
    Collections.sort(tmpfileinfos, new Comparator<FileInfoData>() {
      @Override
      public int compare(FileInfoData lhs, FileInfoData rhs) {
        if(lhs.isDir && !rhs.isDir){
          return 1;
        }

        if(!lhs.isDir && rhs.isDir){
          return -1;
        }

        return lhs.modDate.compareTo(rhs.modDate);
      }
    });

    if(contents != null) {
      contents.clear();
    }
    contents = tmpfileinfos;
  }

  public void setPath(String path){
    if(isSearch){
      return;
    }

    if(path != null && path.length() > 0){
      curPath = path + "/";
      updateFolderInfos(0, true);
    }
  }

  public  FileInfoData getFileInfoOfIndex(int index){
    if(contents == null){
      return null;
    }
    return contents.get(index);
  }

  public String getPathNameOfIndex(int index){
    if(contents == null){
      return null;
    }

    FileInfoData data = contents.get(index);
    return data.path + data.name;
  }

  public int getFileCount(){
    return contents.size();
  }

  public boolean fileExistsAtPath(String path) {
    File file = new File(path);
    return file.exists();
  }

  public boolean createDirectoryAtPath(String name){
    File file = new File(curPath + name);
    return file.mkdir();
  }

  public boolean renameFile(String name, String newname){
    File oldfile = new File(curPath + name);
    File newfile = new File(curPath + newname);
    return oldfile.renameTo(newfile);
  }

  public boolean copyFile(String name, String newname){
    String oldpathname = curPath + name;
    String newpathname = curPath + newname;
    ShellUtils.CommandResult result = ShellUtils.execCommand("cp -rf " + oldpathname + " " + newpathname, false);
    return (result.result == 0);
  }

  //设置拷贝或者剪切的的源路径名
  public void setOrgPathName(String name, int operate){
    if(name != null){
      gOrgPathname = name;
      gOperateType = operate;
    }else{
      gOrgPathname = null;
      gOperateType = -1;
    }
  }

//粘贴操作
  public int pasteFile(){
    String fname = gOrgPathname.substring(gOrgPathname.lastIndexOf("/") + 1);
    String newPathName = curPath + fname;
    if(gOrgPathname.equals(newPathName)){
      return 0;
    }

    File oldfile = new File(gOrgPathname);
    if(oldfile.isDirectory()){
      if(newPathName.length() > gOrgPathname.length()) {
        String tmpPath = newPathName.substring(0, gOrgPathname.length());
        if (tmpPath.equals(gOrgPathname)) {
          return 1;
        }
      }
    }

    ShellUtils.CommandResult result = ShellUtils.execCommand("mv " + gOrgPathname + " " + newPathName, false);
    if(result.result == 0){
      gOrgPathname = null;
      gOperateType = -1;
      return 0;
    }
    return -1;
  }

//获取预定的文件操作路径名称
  public String getOrgFileOperate(int operateType){
    return gOrgPathname;
  }

  public boolean getPasteStatus(){
    if(gOrgPathname == null || gOrgPathname.length() == 0){
      return false;
    }

    String fname = gOrgPathname.substring(gOrgPathname.lastIndexOf("/") + 1);
    String newPathName = curPath + fname;
    if(gOrgPathname.equals(newPathName)){
      return false;
    }

    File oldfile = new File(gOrgPathname);
    if(oldfile.isDirectory()){
      if(newPathName.length() > gOrgPathname.length()) {
        String tmpPath = newPathName.substring(0, gOrgPathname.length());
        if (tmpPath.equals(gOrgPathname)) {
          return false;
        }
      }
    }
    return true;
  }

  public boolean deleteFile(String name){
    String newPathName = curPath + name;
    ShellUtils.CommandResult result = ShellUtils.execCommand("rm -rf " + newPathName, false);
    return (result.result == 0);
  }

  public boolean renameFileByIndex(int index, String newname){
    FileInfoData fileinfo = contents.get(index);
    String path = fileinfo.path.substring(0, fileinfo.path.lastIndexOf("/") + 1);
    String newpathname = path + newname;
    ShellUtils.CommandResult result = ShellUtils.execCommand("mv " + fileinfo.path + " " + newpathname, false);
    boolean ret = (result.result == 0);
    if(ret){
      if(files != null){
        files.remove(fileinfo.tag);
        files.add(newpathname);
      }
      fileinfo.path = newpathname;
      fileinfo.name = newname;
    }
    return ret;
  }

  public boolean deleteFileByIndex(int index){
    FileInfoData fileinfo = contents.get(index);
    ShellUtils.CommandResult result = ShellUtils.execCommand("rm -rf " + fileinfo.path, false);
    if(result.result == 0){
      if(files != null){
        files.remove(fileinfo.tag);
      }
      contents.remove(index);
      return true;
    }
    return false;
  }

  public void clear(){
    if(contents != null){
      contents.clear();
    }

    if(files != null){
      files.clear();
    }
  }
}