//
//  SearchViewController.m
//  FileBrowser
//
//  Created by leeh on 16/9/9.
//  Copyright © 2016年 leeh. All rights reserved.
//
#import "UIKit/UIKit.h"
#import <QuickLook/QuickLook.h>
#import "NSFileManager-Utilities.h"
#import "FileInfoData.h"
#import "FolderData.h"
#import "FilterViewController.h"
#import "UITableView+WLEmptyPlaceHolder.h"

@interface FilterViewController () <UITableViewDataSource, UITableViewDelegate, QLPreviewControllerDataSource>{
  UILongPressGestureRecognizer *longPress;
  FolderData* folder;
  NSInteger curIndex;
  NSArray* sortitems;
  UIView* emptyView;
  JUFileType fileType;
}
@end

@implementation FilterViewController

- (id)initWithFileType:(JUFileType)ftype{
  self = [super initWithStyle:UITableViewStylePlain];
  if(self){
    
    self.tableView.sectionHeaderHeight = 40.0f;
    self.tableView.separatorInset = UIEdgeInsetsMake(0, 0, 0, 5);
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleSingleLine;
  
    self.navigationItem.titleView = nil;
    
    UIButton *backButton = [UIButton buttonWithType:UIButtonTypeCustom];
    [backButton setFrame:CGRectMake(0, 0, 100, 30)];
    [backButton setImage:[UIImage imageNamed:@"back"] forState:UIControlStateNormal];
    
    [backButton setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
    [backButton setTitle:@"分类" forState:UIControlStateNormal];
    [backButton addTarget:self action:@selector(backClicked:) forControlEvents:UIControlEventTouchUpInside];
    
    
    UIBarButtonItem *backButtonItem = [[UIBarButtonItem alloc] initWithCustomView:backButton];
    
    UIBarButtonItem*leftnegativeSpacer = [[UIBarButtonItem alloc]
                                       initWithBarButtonSystemItem:UIBarButtonSystemItemFixedSpace
                                       target:nil action:nil];
    leftnegativeSpacer.width = -20.0f;
    
    fileType = ftype;
    self.navigationItem.leftBarButtonItems = @[leftnegativeSpacer, backButtonItem];
    folder = [[FolderData alloc] initWithPathByFilter:NSDocumentsFolder()
                                     filtertype:fileType];
    
    return self;
  }
  return nil;
}

- (void)viewDidLoad {
  [super viewDidLoad];
}

- (void)viewWillDisappear:(BOOL)animated{
  if ([self.navigationController.viewControllers indexOfObject:self] == NSNotFound){
    //用户点击了返回按钮
    [_delegate refreshContent];
  }
  [super viewWillDisappear:animated];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

- (void)backClicked:(UIBarButtonItem *)backBar{
  [self.navigationController popToViewController:[self.navigationController.viewControllers objectAtIndex:self.navigationController.viewControllers.count - 2]
                                        animated:YES];
}

- (void)renameFile:(NSUInteger)index toFile:(NSString *)newName {
  NSError *error;
  [folder renameFileByIndex:index to:newName error:&error];
   if(error != nil){
     [self showAlert:@"重命名" Message:error.localizedDescription];
     return;
   }
   [self.tableView reloadData];
}

- (void)deleteFile:(NSIndexPath *)indexPath {
  NSError *error;
  if([folder deleteFileByIndex:indexPath.row error:&error]){
    [self.tableView reloadData];
    return;
  }
  
  if(error != nil){
    [self showAlert:@"删除失败：" Message:error.localizedDescription];
  }
}

- (void)renameAlert:(NSIndexPath *)indexPath {
  NSString *filename = [[folder getFileInfoOfIndex:indexPath.row] pathName];
  UIAlertController *renameAlert = [UIAlertController alertControllerWithTitle:@"重命名" message:nil preferredStyle:UIAlertControllerStyleAlert];
  [renameAlert addTextFieldWithConfigurationHandler:^(UITextField *textField) {
    textField.text = filename;
  }];
  
  UIAlertAction *rename = [UIAlertAction actionWithTitle:@"重命名" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
    for(UITextField *textField in renameAlert.textFields) {
      [self renameFile:indexPath.row toFile:textField.text];
    }
  }];
  
  UIAlertAction *cancel = [UIAlertAction actionWithTitle:@"取消" style:UIAlertActionStyleCancel handler:nil];
  
  [renameAlert addAction:rename];
  [renameAlert addAction:cancel];
  
  [self presentViewController:renameAlert animated:YES completion:nil];
}

- (void)showAlert:(NSString *)title Message:(NSString *)message {
  UIAlertController *showAlert = [UIAlertController alertControllerWithTitle:title message:message preferredStyle:UIAlertControllerStyleAlert];
  [self presentViewController:showAlert animated:YES completion:nil];
  dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(2 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
    [self dismissViewControllerAnimated:YES completion:nil];
  });
}

//MARK: Handle LongPressGesture
-(void)handleLongPress:(UILongPressGestureRecognizer *)gestureRecognizer {
  CGPoint p = [gestureRecognizer locationInView:self.tableView];
  NSIndexPath *indexPath = [self.tableView indexPathForRowAtPoint:p];
  if (indexPath != nil) {
    if (gestureRecognizer.state == UIGestureRecognizerStateBegan) {
      [self renameAlert:indexPath];
    }
  }
}


#pragma mark - Table view data source
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  // Return the number of sections.
  return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  // Return the number of rows in the section.
  if(emptyView == nil){
    NSArray *views = [[NSBundle mainBundle] loadNibNamed:@"EmptyView" owner:nil options:nil];
    emptyView = [views lastObject];
  }
  [tableView tableViewDisplayView:emptyView ifNecessaryForRowCount:[folder getFileCount]];
  return [folder getFileCount];
}

//-(void) preViewBtnClick:(id) sender
//{
//  UIButton *button = (UIButton *)sender;
//  NSInteger index = button.tag;
//  
//  QLPreviewController *previewer = [[QLPreviewController alloc] init];
//  // Set data source
//  [previewer setDataSource:self];
//  curIndex = index;
//  [[self navigationController] pushViewController:previewer animated:YES];
//}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  static NSString *CellIdentifier = @"FileCell";
  UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
  
  if (!cell){
    cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
  }
  
  FileInfoData* data = [folder getFileInfoOfIndex:indexPath.row];
  cell.textLabel.text = [[data pathName] lastPathComponent];
  cell.detailTextLabel.text = [NSString stringWithFormat:@"%@   %@", [data size], [data strModDate]];

  switch ([data filetype]) {
    case FileType_TXT:
      cell.imageView.image = [UIImage imageNamed:@"txt"];
      break;
    case FileType_DOC:
      cell.imageView.image = [UIImage imageNamed:@"word"];
      break;
    case FileType_PDF:
      cell.imageView.image = [UIImage imageNamed:@"pdf"];
      break;
    case FileType_PICTURE:
      cell.imageView.image = [UIImage imageNamed:@"picture"];
      break;
    case FileType_MOVIE:
      cell.imageView.image = [UIImage imageNamed:@"movie"];
      break;
    case FileType_MUSIC:
      cell.imageView.image = [UIImage imageNamed:@"music"];
      break;
    case FileType_ZIP:
      cell.imageView.image = [UIImage imageNamed:@"zip"];
      break;
    default:
      cell.imageView.image = [UIImage imageNamed:@"other"];
      break;
  }
  
//  UIImage* img=[UIImage imageNamed:@"preview"];
//  UIButton *btn = [UIButton buttonWithType:UIButtonTypeCustom];
//  btn.frame =CGRectMake(0, 0, 36, 36);
//  [btn setBackgroundImage:img forState:UIControlStateNormal];
//  btn.tag = indexPath.row;
//  [btn addTarget: self action: @selector(preViewBtnClick:) forControlEvents: UIControlEventTouchUpInside];
//  cell.accessoryView = btn;
  
  
  longPress = [[UILongPressGestureRecognizer alloc] initWithTarget:self action:@selector(handleLongPress:)];
  longPress.minimumPressDuration = 1.0; //seconds
  [cell addGestureRecognizer:longPress];
  return cell;
}

-(CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath{
  return TABLEVIEW_ROW_HIGHT;
}


//MARK: UITableViewDelegate
- (void)tableView:(UITableView*)tableView didSelectRowAtIndexPath:(NSIndexPath*)indexPath
{
  //FileInfoData* data = [folder getFileInfoOfIndex:indexPath.row];
  //UIButton *button = (UIButton *)sender;
  //NSInteger index = button.tag;
  
  QLPreviewController *previewer = [[QLPreviewController alloc] init];
  // Set data source
  [previewer setDataSource:self];
  curIndex = indexPath.row;
  [[self navigationController] pushViewController:previewer animated:YES];
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
  if (editingStyle == UITableViewCellEditingStyleDelete) {
    [self deleteFile:indexPath];
  }
}

-(UIView *)InitSectionToolBar{
  UIView *headerView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, self.tableView.frame.size.width, 40)];
  UILabel *messageLabel = [UILabel new];
  switch(fileType){
  case FileType_TXT:
  case FileType_DOC:
  case FileType_PDF:
    messageLabel.text = @"“企业文件中”中的所有文档";
    break;
  case FileType_PICTURE:
    messageLabel.text = @"“企业文件中”中的所有图片";
    break;
  case FileType_MOVIE:
    messageLabel.text = @"“企业文件中”中的所有电影";
    break;
  case FileType_MUSIC:
    messageLabel.text = @"“企业文件中”中的所有音乐";
    break;
  case FileType_ZIP:
    messageLabel.text = @"“企业文件中”中的所有压缩包";
    break;
  default:
    messageLabel.text = @"“企业文件中”中的所有文件";
  }
  
  messageLabel.font = [UIFont preferredFontForTextStyle:UIFontTextStyleBody];
  CGRect frame = messageLabel.frame;
  frame.origin.x += 10;
  frame.origin.y += 10;
  messageLabel.frame = frame;
  messageLabel.textColor = [UIColor blackColor];
  messageLabel.textAlignment = NSTextAlignmentCenter;
  [messageLabel sizeToFit];
  [headerView addSubview:messageLabel];
  [headerView setBackgroundColor:[UIColor colorWithRed:238.0/255.0 green:238.0/255.0 blue:238.0/255.0 alpha:1]];
  return headerView;
}

//方法如下：
- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section {
  //  UIView *headerView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, tableView.frame.size.width, 30)];
  //  UILabel *label = [[UILabel alloc] initWithFrame:CGRectMake(5, 5, tableView.frame.size.width, 20)];
  //  label.font = [UIFont systemFontOfSize:14.0f];  //UILabel的字体大小
  //  label.numberOfLines = 1;  //必须定义这个属性，否则UILabel不会换行
  //  label.textColor = [UIColor whiteColor];
  //  label.textAlignment = NSTextAlignmentLeft;  //文本对齐方式
  //  [label setBackgroundColor:[UIColor clearColor]];
  //
  //  [headerView setBackgroundColor:[UIColor lightGrayColor]];
  //  [headerView addSubview:label];
  //
  //  NSRange range;
  //  range = [self.path rangeOfString:gFirstpath];
  //  if(range.location != NSNotFound) {
  //    label.text = [NSString stringWithFormat:@"[%@]", [self.path substringFromIndex:range.location]];
  //  }
  return [self InitSectionToolBar];
}

-(UITableViewCellEditingStyle)tableView:(UITableView*)tableView editingStyleForRowAtIndexPath:(NSIndexPath*)indexPath {
  return UITableViewCellEditingStyleDelete;
}

#pragma mark - QuickLook

- (BOOL)previewController:(QLPreviewController *)controller shouldOpenURL:(NSURL *)url forPreviewItem:(id <QLPreviewItem>)item {
  return YES;
}

- (NSInteger) numberOfPreviewItemsInPreviewController: (QLPreviewController *) controller {
  return 1;
}

- (id <QLPreviewItem>) previewController: (QLPreviewController *) controller
                      previewItemAtIndex: (NSInteger) index {
  NSString *newPath = [folder getPathNameOfIndex:curIndex];
  return [NSURL fileURLWithPath:newPath];
}

@end
