//
//  FolderData.m
//  FileBrowser
//
//  Created by leeh on 16/9/7.
//  Copyright © 2016年 leeh. All rights reserved.
//

#import "FolderData.h"
#import "FileInfoData.h"

@interface FolderData(){
  NSString* curPath;
  NSMutableArray* contents;
  NSFileManager* fileManager;
  bool isSearch;
  NSMutableArray* files;
}
@end

@implementation FolderData

static NSString* gOrgPathname = nil;
static NSInteger gOperateType = -1; //0为拷贝，1为剪切

-(NSArray*)getFolderInfos{
  return contents;
}

- (id)initWithPath:(NSString *)path
          isSearch:(bool)bSearch;
{
  self = [super init];
  if(self){
    curPath = path;
    contents = [NSMutableArray array];
    fileManager = [NSFileManager defaultManager];
    isSearch = bSearch;
    if(isSearch){
      NSString *file;
      NSMutableArray *results = [NSMutableArray array];
      NSDirectoryEnumerator *dirEnum = [fileManager enumeratorAtPath:curPath];
      while((file = [dirEnum nextObject])){
        BOOL isDir;
        [fileManager fileExistsAtPath:[curPath stringByAppendingPathComponent:file] isDirectory: &isDir];
        if(!isDir){
          [results addObject:file];
        }
      }
      files = results;
    }
  }
  return self;
}

- (id)initWithPathByFilter:(NSString *)path
          filtertype:(JUFileType)type
{
  self = [super init];
  if(self){
    curPath = path;
    contents = [NSMutableArray array];
    fileManager = [NSFileManager defaultManager];
    
    NSString *file;
    NSMutableArray *results = [NSMutableArray array];
    NSDirectoryEnumerator *dirEnum = [fileManager enumeratorAtPath:curPath];
    while((file = [dirEnum nextObject])){
      BOOL isDir;
      [fileManager fileExistsAtPath:[curPath stringByAppendingPathComponent:file] isDirectory: &isDir];
      if(!isDir){
        [results addObject:file];
      }
    }
    
    NSMutableArray* tmpfileinfos = [NSMutableArray array];
    for(NSString* name in results){
      FileInfoData* data = [self getFileInfoObject:name];
      if(type == FileType_PICTURANDEMOVIE &&
         (data.filetype == FileType_PICTURE || data.filetype == FileType_MOVIE)){
        data.pathName = [curPath stringByAppendingPathComponent:name];
        [tmpfileinfos addObject:data];
      } else if(data.filetype == type || type == 0){
        [tmpfileinfos addObject:data];
      }
    }
    contents = tmpfileinfos;
  }
  return self;
}

-(NSString *)sizeFormatted:(NSNumber *)size
{
  //TODO if OS X 10.8 or iOS 6
  //return [NSByteCountFormatter stringFromByteCount:[size intValue] countStyle:NSByteCountFormatterCountStyleFile];
  
  double convertedValue = [size doubleValue];
  int multiplyFactor = 0;
  
  NSArray *tokens = @[@"B", @"KB", @"MB", @"GB", @"TB"];
  
  while(convertedValue > 1024){
    convertedValue /= 1024;
    
    multiplyFactor++;
  }
  
  NSString *sizeFormat = ((multiplyFactor > 1) ? @"%4.2f%@" : @"%4.0f%@");
  
  return [NSString stringWithFormat:sizeFormat, convertedValue, tokens[multiplyFactor]];
}


-(FileInfoData *) getFileInfoObject:(NSString*)name{
  FileInfoData *infodata = [[FileInfoData alloc] init];
  BOOL isDirectory;
  NSString* pathname = [curPath stringByAppendingPathComponent:name];
  [fileManager fileExistsAtPath:pathname isDirectory:&isDirectory];
  NSError *error = nil;
  
  if(isDirectory){
    infodata.isDir = true;
    infodata.pathName = name;
    
    NSArray *tempFiles = [fileManager contentsOfDirectoryAtPath:pathname error:&error];
    infodata.subcount = [tempFiles count];
  }else{
    infodata.isDir = false;
    infodata.pathName = name;
    infodata.subcount = 0;
    
    NSString* ext = [[name pathExtension] lowercaseString];
    if([ext isEqualToString:@"png"] || [ext isEqualToString:@"jpg"] || [ext isEqualToString:@"jpeg"] || [ext isEqualToString:@"bmp"] || [ext isEqualToString:@"gif"]){
      infodata.filetype = FileType_PICTURE;
    }else if([ext isEqualToString:@"mp4"]){
      infodata.filetype = FileType_MOVIE;
    }else if([ext isEqualToString:@"pdf"]){
      infodata.filetype = FileType_PDF;
    }else if([ext isEqualToString:@"zip"] || [ext isEqualToString:@"gzip"] || [ext isEqualToString:@"rar"] || [ext isEqualToString:@"bz2"]){
      infodata.filetype = FileType_ZIP;
    }else if([ext isEqualToString:@"doc"] || [ext isEqualToString:@"docx"]){
      infodata.filetype = FileType_DOC;
    }else if([ext isEqualToString:@"mp3"]){
      infodata.filetype = FileType_MUSIC;
    }else if([ext isEqualToString:@"txt"] || [ext isEqualToString:@"ini"] || [ext isEqualToString:@"rtf"] ){
      infodata.filetype = FileType_DOC;
    }else{
      infodata.filetype = FileType_OTHER;
    }
  }
  
  infodata.size = nil;
  infodata.nSize = 0;
  NSDictionary* fileAttributes = [fileManager attributesOfItemAtPath:pathname error:&error];
  if(fileAttributes != nil) {
    infodata.modDate = [fileAttributes objectForKey:NSFileModificationDate];
    
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
    infodata.strModDate = [dateFormatter stringFromDate:infodata.modDate];
    if(!isDirectory){
      infodata.nSize = [[fileAttributes objectForKey:NSFileSize] integerValue];
      infodata.size = [self sizeFormatted:[fileAttributes objectForKey:NSFileSize]];
    }
  }
  return infodata;
}

- (void)setPath:(NSString *)path{
  if(isSearch){
    return;
  }
  
  if(path != nil){
    curPath = path;
    [self updateFolderInfos:0];
  }  
}

-(FileInfoData*)getFileInfoOfIndex:(NSInteger)index{
  if(contents == nil || contents.count == 0){
    return nil;
  }
  return contents[index];
}

-(NSString*)getPathNameOfIndex:(NSInteger)index{
  if(contents == nil){
    return nil;
  }
  
  return [curPath stringByAppendingPathComponent:[contents[index] pathName]];
}

-(NSInteger)getFileCount{
  return [contents count];
}

- (void)updateFolderInfos:(NSUInteger) sorttype{
  if(isSearch){
    return;
  }
  
  NSError *error = nil;
  NSArray *tempFiles = [fileManager contentsOfDirectoryAtPath:curPath error:&error];
  if(sorttype == 0){
    NSArray* tempsortFiles = [tempFiles sortedArrayWithOptions:0 usingComparator:^NSComparisonResult(NSString* file1, NSString* file2) {
      NSString* newPath1 = [curPath stringByAppendingPathComponent:file1];
      NSString* newPath2 = [curPath stringByAppendingPathComponent:file2];
      
      BOOL isDirectory1, isDirectory2;
      [fileManager fileExistsAtPath:newPath1 isDirectory:&isDirectory1];
      [fileManager fileExistsAtPath:newPath2 isDirectory:&isDirectory2];
      
      if (isDirectory1 && !isDirectory2) {
        return NSOrderedDescending;
      }
      return NSOrderedAscending;
    }];
    
    NSMutableArray* tmpfileinfos = [NSMutableArray array];
    for(NSString* name in tempsortFiles){
      FileInfoData* data = [self getFileInfoObject:name];
      [tmpfileinfos addObject:data];
    }
    contents = tmpfileinfos;
    return;
  }
  
  //时间排序
  NSMutableArray* tmpfileinfos = [NSMutableArray array];
  for(NSString* name in tempFiles){
    FileInfoData* data = [self getFileInfoObject:name];
    //if(data.isDir || filetype == 0 || filetype == data.filetype){
    [tmpfileinfos addObject:data];
    //}
  }
  
  NSArray* tempsortFiles = [tmpfileinfos sortedArrayWithOptions:0 usingComparator:^NSComparisonResult(FileInfoData* obj1, FileInfoData* obj2) {
    if(obj1.isDir && !obj2.isDir) {
      return NSOrderedDescending;
    }
    
    if((!obj1.isDir && !obj2.isDir) || (obj1.isDir && obj2.isDir)){
      return [obj2.modDate compare:obj1.modDate];
    }
    return NSOrderedAscending;
  }];  
  contents = [tempsortFiles mutableCopy];
}

//文件搜索
- (void)searchFolderInfos:(NSString*)searchstr
                 sortType:(NSUInteger) sorttype{
  
  NSMutableArray* tmpfileinfos = [NSMutableArray array];
  int i = 0;
  if(searchstr != nil && searchstr.length > 0){
    for(NSString* name in files){
      NSString* lastname = [name lastPathComponent];
      NSRange range = [lastname rangeOfString:searchstr];
      if(range.location != NSNotFound){
        FileInfoData* data = [self getFileInfoObject:name];
        data.tag = i;
        [tmpfileinfos addObject:data];
      }
      i++;
    }
  }else{
    if([contents count] == 0){
      for(NSString* name in files){
        FileInfoData* data = [self getFileInfoObject:name];
        data.tag = i++;
        [tmpfileinfos addObject:data];
      }
    }else{
      tmpfileinfos = contents;
    }
  }
  
  
  if(sorttype == 0){
    NSArray* tempsortFiles = [tmpfileinfos sortedArrayWithOptions:0 usingComparator:^NSComparisonResult(FileInfoData* obj1, FileInfoData* obj2) {
        NSString* lastname1 = [obj1.pathName lastPathComponent];
        NSString* lastname2 = [obj2.pathName lastPathComponent];
        return [lastname1 compare:lastname2];
    }];
    contents = [tempsortFiles mutableCopy];
    return;
  }
  
  //时间排
  NSArray* tempsortFiles = [tmpfileinfos sortedArrayWithOptions:0 usingComparator:^NSComparisonResult(FileInfoData* obj1, FileInfoData* obj2) {
      return [obj2.modDate compare:obj1.modDate];
  }];
  contents = [tempsortFiles mutableCopy];
}


-(bool)createDirectoryAtPath:(NSString *)name
                       error:(NSError **)error{
  
  if(isSearch){
    return false;
  }
  
  NSString *pathname = [curPath stringByAppendingPathComponent:name];
  return [fileManager createDirectoryAtPath:pathname withIntermediateDirectories:NO attributes:nil error:error];
}

-(bool)fileExistsAtPath:(NSString *)name{
  NSString *pathname = [curPath stringByAppendingPathComponent:name];
  return [fileManager fileExistsAtPath:pathname];
}

- (bool)renameFile:(NSString *)oldname
            to:(NSString *)newname
             error:(NSError **)error{
  
  NSString *oldPathName = [curPath stringByAppendingPathComponent:oldname];
  NSString *newPathName = [curPath stringByAppendingPathComponent:newname];
  return [fileManager moveItemAtPath:oldPathName toPath:newPathName error:error];
}

- (BOOL)renameFileByIndex:(NSInteger)index
                to:(NSString *)newname
             error:(NSError **)error{
  if(files){
    NSString *oldPathName = [curPath stringByAppendingPathComponent:[contents[index] pathName]];
    NSString* path = [oldPathName stringByDeletingLastPathComponent];
    NSString *newPathName = [path stringByAppendingPathComponent:newname];
    [fileManager moveItemAtPath:oldPathName toPath:newPathName error:error];
    
    if(*error == nil){
      FileInfoData* data = contents[index];
      int fileindex = data.tag;
      files[fileindex] = [[[contents[index] pathName] stringByDeletingLastPathComponent] stringByAppendingPathComponent:newname];
      FileInfoData* fileinfo = contents[index];
      fileinfo.pathName = files[fileindex];
      return true;
    }
  }
  return false;
}

-(bool)copyFile:(NSString *)name
            new:(NSString *)newname
          error:(NSError **)error
{
  if(isSearch){
    return false;
  }
  
  NSString *oldPath = [curPath stringByAppendingPathComponent:name];
  NSString *newPath = [curPath stringByAppendingPathComponent:newname];
  if(![fileManager fileExistsAtPath:newPath]) {
    return [fileManager copyItemAtPath:oldPath toPath:newPath error:error];
  }
  return false;
}

-(void)setOrgPathName:(NSString *)name
           operateTye:(NSInteger)operate{
  if(name != nil){
    gOrgPathname = [curPath stringByAppendingPathComponent:name];
    gOperateType = operate;
  }else{
    gOrgPathname = nil;
    gOperateType = -1;
  }
}

-(bool)pasteFile:(NSError **)error
{
  if(gOrgPathname == nil){
    return true;
  }
  
  NSString* name = [gOrgPathname lastPathComponent];
  NSString* newPath = [curPath stringByAppendingPathComponent:name];
  if(![fileManager fileExistsAtPath:newPath]) {
    [fileManager copyItemAtPath:gOrgPathname toPath:newPath error:error];
    if(*error){
      return false;
    }
    
    if(gOperateType == 1){
      [fileManager removeItemAtPath:gOrgPathname error:error];
      if(*error == nil){
        gOrgPathname = nil;
        gOperateType = -1;
        return true;
      }
    }
  }else{
    gOrgPathname = nil;
    gOperateType = -1;
    return true;
  }
  return false;
}

-(NSString*)getOrgFileOperate:(NSInteger *)operateType{
  *operateType = gOperateType;
  return gOrgPathname;
}

-(bool)deleteFile:(NSString *)name
              error:(NSError **)error
{
  NSString *newPath = [curPath stringByAppendingPathComponent:name];
  if ([fileManager fileExistsAtPath:newPath]) {
    return [fileManager removeItemAtPath:newPath error:error];
  }
  return true;
}

-(bool)deleteFileByIndex:(NSUInteger)index
            error:(NSError **)error
{
  FileInfoData* fileinfo = contents[index];
  NSString *newPath = [curPath stringByAppendingPathComponent:[fileinfo pathName]];
  if([fileManager fileExistsAtPath:newPath]) {
    [fileManager removeItemAtPath:newPath error:error];
  }
  
  if(*error == nil){
    if(files){
      [files removeObjectAtIndex:fileinfo.tag];
    }
    [contents removeObjectAtIndex:index];
    return true;
  }
  return false;
}


@end
