//
//  TZImagePickerController.m
//  TZImagePickerController
//
//  Created by 谭真 on 15/12/24.
// modify by lihong on 16/9/26
//  Copyright © 2015年 谭真. All rights reserved.
//  version 1.6.4 - 2016.09.05

#import "TZImagePickerController.h"
#import "TZPhotoPickerController.h"
#import "TZAssetModel.h"
#import "TZAssetCell.h"
#import "UIView+Layout.h"
#import "TZImageManager.h"
#import "FileInfoData.h"
#import "FileBrowserPlugin.h"

@interface TZImagePickerController () {
    NSTimer *_timer;
    UILabel *_tipLable;
    UIButton *_settingBtn;
    BOOL _pushToPhotoPickerVc;
    UIStatusBarStyle _originStatusBarStyle;
    UIButton *_okButton;
  
    __block void (^MyVideoHanldeBlock)(NSString *outputPath);
}

@property (nonatomic, assign) NSInteger columnNumber;
@end


@implementation TZImagePickerController

static NSMutableArray *resultStrings;
static NSUInteger curSelectNumber;
static NSMutableArray<TZAssetModel *> *selectedVideoModels;
static bool inHandleVideo;

- (void)okButtonClick {
  curSelectNumber = self.selectedModels.count;
  if(curSelectNumber == 0){
    return;
  }
  
  inHandleVideo = false;
  [FileBrowserPlugin showProgressHUD:self.timeout];
  if(resultStrings == nil){
    resultStrings = [NSMutableArray array];
    selectedVideoModels = [NSMutableArray array];
  }else{
    [resultStrings removeAllObjects];
    [selectedVideoModels removeAllObjects];
  }

  NSString* docsPath = [NSTemporaryDirectory() stringByStandardizingPath];
  NSFileManager* fileMgr = [[NSFileManager alloc] init];
  
  [TZImageManager manager].shouldFixOrientation = YES;
 
  for (NSInteger i = 0; i < self.selectedModels.count; i++) {
    TZAssetModel *model = self.selectedModels[i];
    if([model.asset isKindOfClass:[FileInfoData class]]){
      FileInfoData* data = (FileInfoData*)model.asset;
      [resultStrings addObject:data.pathName];
      curSelectNumber--;
      continue;
    }
    
    if(model.type == TZAssetModelMediaTypePhoto){
      [[TZImageManager manager] getPhotoWithAsset:model.asset completion:^(UIImage *photo, NSDictionary *info, BOOL isDegraded) {
        if(isDegraded){
          NSLog(@"----isDegraded------");
          return;
        }else{
          NSLog(@"----not Degraded------");
        }
        
        NSString* filePath;
        if(photo) {
            photo = [TZImageManager scaleImage:photo toSize:CGSizeMake(self.photoWidth, (int)(self.photoWidth * photo.size.height / photo.size.width))];
            int i = 1;
            do{
              filePath = [NSString stringWithFormat:@"%@/%03d.%@", docsPath, i++, @"jpg"];
            }while([fileMgr fileExistsAtPath:filePath]);
            
            NSError* err = nil;
            NSData* data = UIImageJPEGRepresentation(photo, 0.9f);
            if([data writeToFile:filePath options:NSAtomicWrite error:&err]) {
              [resultStrings addObject:filePath];
            }
            curSelectNumber--;
            if(curSelectNumber <= 0){
              [FileBrowserPlugin hideProgressHUD];
              if(_plugPickerDelegate){
                [_plugPickerDelegate selected:resultStrings];
                curSelectNumber = 9;
              }
            }
        }
      }];
      continue;
    }
    
    if(inHandleVideo == true){
      [selectedVideoModels addObject:model];
      continue;
    }
    
    inHandleVideo = true;
    
    __block TZImagePickerController *blockSelf = self;
    MyVideoHanldeBlock = ^(NSString *outputPath){
      NSLog(@"---getVideoOutputPathWithAsset %@---", outputPath);
      
      __block TZImagePickerController *newblockSelf = blockSelf;
      
      if(outputPath != nil && outputPath.length > 5){
        [resultStrings addObject:outputPath];
      }else{
        NSLog(@"---getVideoOutputPathWithAsset failure index = %lu---", curSelectNumber - 1);
        [FileBrowserPlugin showAlertWithTitle:newblockSelf.navigationController title:@"导出视频失败"];
      }
      
      curSelectNumber--;
      NSLog(@"uncomplete %lu", (unsigned long)curSelectNumber);
      if(curSelectNumber <= 0){
        [FileBrowserPlugin hideProgressHUD];
        if(newblockSelf->_plugPickerDelegate){
          [newblockSelf->_plugPickerDelegate selected:resultStrings];
          curSelectNumber = 9;
        }
        return;
      }
      
      if([selectedVideoModels count] > 0){
        TZAssetModel *model = selectedVideoModels[0];
        [selectedVideoModels removeObjectAtIndex:0];
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.2f * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
          [[TZImageManager manager] getVideoOutputPathWithAsset:model.asset completion:newblockSelf->MyVideoHanldeBlock];
        });
      }else{
        inHandleVideo = false;
      }
    };
    
    NSLog(@"selectedVideoModels ＝ 0 uncomplete %lu", (unsigned long)curSelectNumber);
    [[TZImageManager manager] getVideoOutputPathWithAsset:model.asset completion:blockSelf->MyVideoHanldeBlock];
  }
  
  if(curSelectNumber <= 0){
    [FileBrowserPlugin hideProgressHUD];
    if(_plugPickerDelegate){
      [_plugPickerDelegate selected:resultStrings];
      curSelectNumber = 9;
    }
  }
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor whiteColor];
    self.navigationBar.translucent = YES;
    [TZImageManager manager].shouldFixOrientation = NO;

    // Default appearance, you can reset these after this method
    // 默认的外观，你可以在这个方法后重置
    self.oKButtonTitleColorNormal   = [UIColor colorWithRed:0.157 green:0.686 blue:0.973 alpha:1.0];
    self.oKButtonTitleColorDisabled = [UIColor colorWithRed:0.157 green:0.686 blue:0.973 alpha:0.5];
  
    _okButton = [UIButton buttonWithType:UIButtonTypeCustom];
    _okButton.frame = CGRectMake(self.view.tz_width - 80 - 12, 3, 80, 44);
    _okButton.contentHorizontalAlignment = UIControlContentHorizontalAlignmentRight;
    _okButton.titleLabel.font = [UIFont systemFontOfSize:16];
    [_okButton addTarget:self action:@selector(okButtonClick) forControlEvents:UIControlEventTouchUpInside];
    [_okButton setTitleColor:_oKButtonTitleColorNormal forState:UIControlStateNormal];
    [_okButton setTitleColor:_oKButtonTitleColorDisabled forState:UIControlStateDisabled];
    _okButton.enabled = _selectedModels.count;
  
    UIView *divide = [[UIView alloc] init];
    CGFloat rgb2 = 222 / 255.0;
    divide.backgroundColor = [UIColor colorWithRed:rgb2 green:rgb2 blue:rgb2 alpha:1.0];
    divide.frame = CGRectMake(0, 0, self.view.tz_width, 1);
  
    UIView *bottomToolBar = [[UIView alloc] initWithFrame:CGRectMake(0, self.view.tz_height - 50, self.view.tz_width, 50)];
    CGFloat rgb = 253 / 255.0;
    bottomToolBar.backgroundColor = [UIColor colorWithRed:rgb green:rgb blue:rgb alpha:1.0];

    [bottomToolBar addSubview:divide];
    [bottomToolBar addSubview:_okButton];
    [self.view addSubview:bottomToolBar];
}

- (void)setBarItemTextFont:(UIFont *)barItemTextFont {
    _barItemTextFont = barItemTextFont;
    [self configBarButtonItemAppearance];
}

- (void)setBarItemTextColor:(UIColor *)barItemTextColor {
    _barItemTextColor = barItemTextColor;
    [self configBarButtonItemAppearance];
}

- (void)configBarButtonItemAppearance {
    UIBarButtonItem *barItem;
    if (iOS9Later) {
        barItem = [UIBarButtonItem appearanceWhenContainedInInstancesOfClasses:@[[TZImagePickerController class]]];
    } else {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
        barItem = [UIBarButtonItem appearanceWhenContainedIn:[TZImagePickerController class], nil];
#pragma clang diagnostic pop
    }
    NSMutableDictionary *textAttrs = [NSMutableDictionary dictionary];
    textAttrs[NSForegroundColorAttributeName] = self.barItemTextColor;
    textAttrs[NSFontAttributeName] = self.barItemTextFont;
    [barItem setTitleTextAttributes:textAttrs forState:UIControlStateNormal];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    _originStatusBarStyle = [UIApplication sharedApplication].statusBarStyle;
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
    [UIApplication sharedApplication].statusBarStyle = iOS7Later ? UIStatusBarStyleLightContent : UIStatusBarStyleBlackOpaque;
#pragma clang diagnostic pop
    [self refreshBottomToolBarStatus];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [UIApplication sharedApplication].statusBarStyle = _originStatusBarStyle;
    [FileBrowserPlugin hideProgressHUD];
}

- (instancetype)initWithMaxImagesCount:(NSInteger)maxImagesCount delegate:(id<TZImagePickerControllerDelegate>)delegate {
    return [self initWithMaxImagesCount:maxImagesCount columnNumber:4 delegate:delegate];
}

- (instancetype)initWithMaxImagesCount:(NSInteger)maxImagesCount columnNumber:(NSInteger)columnNumber delegate:(id<TZImagePickerControllerDelegate>)delegate {
    TZAlbumPickerController *albumPickerVc = [[TZAlbumPickerController alloc] init];
    albumPickerVc.columnNumber = columnNumber;
    self = [super initWithRootViewController:albumPickerVc];
    if (self) {
        self.maxImagesCount = maxImagesCount > 0 ? maxImagesCount : 9; // Default is 9 / 默认最大可选9张图片
        self.pickerDelegate = delegate;
        self.selectedModels = [NSMutableArray array];
        
        // Allow user picking original photo and video, you also can set No after this method
        // 默认准许用户选择原图和视频, 你也可以在这个方法后置为NO
        self.allowPickingOriginalPhoto = YES;
        self.allowPickingVideo = YES;
        self.allowPickingImage = YES;
        self.timeout = 15;
        self.photoWidth = 828.0;
        self.photoPreviewMaxWidth = 600;
        self.sortAscendingByModificationDate = YES;
        self.autoDismiss = YES;
        self.barItemTextFont = [UIFont systemFontOfSize:15];
        self.barItemTextColor = [UIColor colorWithRed:0.157 green:0.686 blue:0.973 alpha:1];
        self.columnNumber = columnNumber;
        [self configDefaultImageName];
        
        if (![[TZImageManager manager] authorizationStatusAuthorized]) {
            _tipLable = [[UILabel alloc] init];
            _tipLable.frame = CGRectMake(8, 120, self.view.tz_width - 16, 60);
            _tipLable.textAlignment = NSTextAlignmentCenter;
            _tipLable.numberOfLines = 0;
            _tipLable.font = [UIFont systemFontOfSize:16];
            _tipLable.textColor = [UIColor blackColor];
            NSString *appName = [[NSBundle mainBundle].infoDictionary valueForKey:@"CFBundleDisplayName"];
            if (!appName) appName = [[NSBundle mainBundle].infoDictionary valueForKey:@"CFBundleName"];
            NSString *tipText = [NSString stringWithFormat:@"打开允许 %@ 访问相册 \"设置 -> 隐私 -> 图片\"", appName];
            _tipLable.text = tipText;
            [self.view addSubview:_tipLable];
            
            _settingBtn = [UIButton buttonWithType:UIButtonTypeSystem];
            [_settingBtn setTitle:@"设置" forState:UIControlStateNormal];
            _settingBtn.frame = CGRectMake(0, 180, self.view.tz_width, 44);
            _settingBtn.titleLabel.font = [UIFont systemFontOfSize:18];
            [_settingBtn addTarget:self action:@selector(settingBtnClick) forControlEvents:UIControlEventTouchUpInside];
            [self.view addSubview:_settingBtn];
            
            _timer = [NSTimer scheduledTimerWithTimeInterval:0.2 target:self selector:@selector(observeAuthrizationStatusChange) userInfo:nil repeats:YES];
        }
    }
    return self;
}


- (void)configDefaultImageName {
    self.photoSelImageName = @"photo_sel_photoPickerVc.png";
    self.photoDefImageName = @"photo_def_photoPickerVc.png";
}

- (void)observeAuthrizationStatusChange {
    if ([[TZImageManager manager] authorizationStatusAuthorized]) {
        [_tipLable removeFromSuperview];
        [_settingBtn removeFromSuperview];
        [_timer invalidate];
        _timer = nil;
    }
}

- (void)setTimeout:(NSInteger)timeout {
    _timeout = timeout;
    if (timeout < 5) {
        _timeout = 5;
    } else if (_timeout > 60) {
        _timeout = 60;
    }
}

- (void)setColumnNumber:(NSInteger)columnNumber {
    _columnNumber = columnNumber;
    if (columnNumber <= 2) {
        _columnNumber = 2;
    } else if (columnNumber >= 6) {
        _columnNumber = 6;
    }
    
    TZAlbumPickerController *albumPickerVc = [self.childViewControllers firstObject];
    albumPickerVc.columnNumber = _columnNumber;
    [TZImageManager manager].columnNumber = _columnNumber;
}

- (void)setPhotoPreviewMaxWidth:(CGFloat)photoPreviewMaxWidth {
    _photoPreviewMaxWidth = photoPreviewMaxWidth;
    if (photoPreviewMaxWidth > 800) {
        _photoPreviewMaxWidth = 800;
    } else if (photoPreviewMaxWidth < 500) {
        _photoPreviewMaxWidth = 500;
    }
    [TZImageManager manager].photoPreviewMaxWidth = _photoPreviewMaxWidth;
}

- (void)setSelectedAssets:(NSMutableArray *)selectedAssets {
    _selectedAssets = selectedAssets;
    _selectedModels = [NSMutableArray array];
    for (id asset in selectedAssets) {
        TZAssetModel *model = [TZAssetModel modelWithAsset:asset type:TZAssetModelMediaTypePhoto];
        model.isSelected = YES;
        [_selectedModels addObject:model];
    }
}

#pragma mark -RefreshBarStatusDelegate

- (void)refreshBottomToolBarStatus {
  _okButton.enabled = _selectedModels.count > 0;
  NSString* retTitle = _retLabelTitle ? _retLabelTitle:@"发送";
  NSString* okButtonTitle = [NSString stringWithFormat:@"%@(%ld/%ld)", retTitle, (unsigned long)_selectedModels.count, (long)_maxImagesCount];
  if(_okButton.enabled){
    [_okButton setTitle:okButtonTitle forState:UIControlStateNormal];
  }else{
    [_okButton setTitle:retTitle forState:UIControlStateNormal];
  }
}

- (void)setAllowPickingImage:(BOOL)allowPickingImage {
    _allowPickingImage = allowPickingImage;
    NSString *allowPickingImageStr = _allowPickingImage ? @"1" : @"0";
    [[NSUserDefaults standardUserDefaults] setObject:allowPickingImageStr forKey:@"tz_allowPickingImage"];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (void)setAllowPickingVideo:(BOOL)allowPickingVideo {
    _allowPickingVideo = allowPickingVideo;
    NSString *allowPickingVideoStr = _allowPickingVideo ? @"1" : @"0";
    [[NSUserDefaults standardUserDefaults] setObject:allowPickingVideoStr forKey:@"tz_allowPickingVideo"];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (void)setSortAscendingByModificationDate:(BOOL)sortAscendingByModificationDate {
    _sortAscendingByModificationDate = sortAscendingByModificationDate;
    [TZImageManager manager].sortAscendingByModificationDate = sortAscendingByModificationDate;
}

- (void)settingBtnClick {
    if (iOS8Later) {
        [[UIApplication sharedApplication] openURL:[NSURL URLWithString:UIApplicationOpenSettingsURLString]];
    } else {
        [[UIApplication sharedApplication] openURL:[NSURL URLWithString:@"prefs:root=Privacy&path=PHOTOS"]];
    }
}

- (void)pushViewController:(UIViewController *)viewController animated:(BOOL)animated {
    if (iOS7Later) viewController.automaticallyAdjustsScrollViewInsets = NO;
    if (_timer) { [_timer invalidate]; _timer = nil;}
    
    if (self.childViewControllers.count > 0) {
        UIButton *backButton = [[UIButton alloc] initWithFrame:CGRectMake(3, 0, 50, 44)];
        [backButton setImage:[UIImage imageNamedFromMyBundle:@"navi_back.png"] forState:UIControlStateNormal];
        backButton.imageEdgeInsets = UIEdgeInsetsMake(0, -5, 0, 0);
        [backButton setTitle:@"返回" forState:UIControlStateNormal];
        backButton.titleLabel.font = [UIFont systemFontOfSize:15];
        [backButton addTarget:self action:@selector(popViewControllerAnimated:) forControlEvents:UIControlEventTouchUpInside];
        UIBarButtonItem* backButtonItem = [[UIBarButtonItem alloc] initWithCustomView:backButton];
        
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
        backButtonItem.style = UIBarButtonItemStyleBordered;
#pragma clang diagnostic pop
        self.topViewController.navigationItem.backBarButtonItem = backButtonItem;
    }
    [super pushViewController:viewController animated:animated];
}

@end


@interface TZAlbumPickerController ()<UITableViewDataSource,UITableViewDelegate> {
    UITableView *_tableView;
}
@property (nonatomic, strong) NSMutableArray *albumArr;
@end

@implementation TZAlbumPickerController

-(void)cancelSelect{
  TZImagePickerController *imagePickerVc = (TZImagePickerController *)self.navigationController;
  if(imagePickerVc.plugPickerDelegate){
    [imagePickerVc.plugPickerDelegate cancelSelect];
  }
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor whiteColor];
    self.navigationItem.title = @"图片视频";
    self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"取消" style:UIBarButtonItemStylePlain target:self action:@selector(cancelSelect)];
    [self configTableView];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    TZImagePickerController *imagePickerVc = (TZImagePickerController *)self.navigationController;
    [FileBrowserPlugin hideProgressHUD];
    if (_albumArr) {
        for (TZAlbumModel *albumModel in _albumArr) {
            albumModel.selectedModels = imagePickerVc.selectedModels;
        }
        [_tableView reloadData];
    } else {
        [self configTableView];
    }
}

- (void)configTableView {
    TZImagePickerController *imagePickerVc = (TZImagePickerController *)self.navigationController;
    [[TZImageManager manager] getAllAlbums:imagePickerVc.allowPickingVideo allowPickingImage:imagePickerVc.allowPickingImage completion:^(NSArray<TZAlbumModel *> *models) {
        _albumArr = [NSMutableArray arrayWithArray:models];
        for (TZAlbumModel *albumModel in _albumArr) {
            albumModel.selectedModels = imagePickerVc.selectedModels;
        }
      
        if (!_tableView) {
            CGFloat top = 44;
            if (iOS7Later) top += 20;
            _tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, top, self.view.tz_width, self.view.tz_height - top) style:UITableViewStylePlain];
            _tableView.rowHeight = 70;
            _tableView.tableFooterView = [[UIView alloc] init];
            _tableView.dataSource = self;
            _tableView.delegate = self;
            [_tableView registerClass:[TZAlbumCell class] forCellReuseIdentifier:@"TZAlbumCell"];
            [self.view addSubview:_tableView];
          
            CGRect frame = _tableView.frame;
            NSUInteger count = _albumArr.count;
            if(count * 70 > frame.size.height - 50){
              UIEdgeInsets insets = _tableView.contentInset;
              _tableView.contentInset = UIEdgeInsetsMake(insets.top, insets.left, insets.bottom + 49, insets.right);
            }
        } else {
            [_tableView reloadData];
        }
    }];
}

#pragma mark - UITableViewDataSource && Delegate

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return _albumArr.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    TZAlbumCell *cell = [tableView dequeueReusableCellWithIdentifier:@"TZAlbumCell"];
    TZImagePickerController *imagePickerVc = (TZImagePickerController *)self.navigationController;
    cell.selectedCountButton.backgroundColor = imagePickerVc.oKButtonTitleColorNormal;
    cell.model = _albumArr[indexPath.row];
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    TZPhotoPickerController *photoPickerVc = [[TZPhotoPickerController alloc] init];
  
    photoPickerVc.refreshStatusDelegate = (TZImagePickerController *)self.navigationController;
  
    photoPickerVc.columnNumber = self.columnNumber;
    TZAlbumModel *model = _albumArr[indexPath.row];
    photoPickerVc.model = model;
    __weak TZAlbumPickerController *weakSelf = self;
    [photoPickerVc setBackButtonClickHandle:^(TZAlbumModel *model) {
        [weakSelf.albumArr replaceObjectAtIndex:indexPath.row withObject:model];
    }];
  
    [self.navigationController pushViewController:photoPickerVc animated:YES];
    [tableView deselectRowAtIndexPath:indexPath animated:NO];
}
@end
