//
//  TZPhotoPickerController.m
//
//  Created by 谭真 on 15/12/24.
// modify by lihong on 16/9/23
//  Copyright © 2015年 谭真. All rights reserved.
//

#import "TZPhotoPickerController.h"
#import "TZImagePickerController.h"
#import "TZAssetCell.h"
#import "TZAssetModel.h"
#import "UIView+Layout.h"
#import "TZImageManager.h"


@interface TZPhotoPickerController ()<UICollectionViewDataSource, UICollectionViewDelegate, UIImagePickerControllerDelegate,UINavigationControllerDelegate, UIAlertViewDelegate>{
    NSMutableArray *_models;
    BOOL _shouldScrollToBottom;
    TZImagePickerController *_imagePickerVc;
}

@property (nonatomic, strong) UICollectionView *collectionView;
//@property (nonatomic, strong) UIImagePickerController *imagePickerVc;
@end

static CGSize AssetGridThumbnailSize;

@implementation TZPhotoPickerController

- (void)viewDidLoad {
    [super viewDidLoad];
    _shouldScrollToBottom = YES;
    self.view.backgroundColor = [UIColor whiteColor];
    self.navigationItem.title = _model.name;
    self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"取消" style:UIBarButtonItemStylePlain target:self action:@selector(cancel)];
    _models = [NSMutableArray arrayWithArray:_model.models];
    [self initSubviews];
}

- (void)initSubviews {
  [self configCollectionView];
  [self checkSelectedModels];
}

- (void)viewWillDisappear:(BOOL)animated {
  [super viewWillDisappear:animated];
//更新拍照信息
  if (self.backButtonClickHandle) {
    self.backButtonClickHandle(_model);
  }
}

- (void)configCollectionView {
    _imagePickerVc = (TZImagePickerController *)self.navigationController;
    UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
    CGFloat margin = 5;
    CGFloat itemWH = (self.view.tz_width - (self.columnNumber + 1) * margin) / self.columnNumber;
    layout.itemSize = CGSizeMake(itemWH, itemWH);
    layout.minimumInteritemSpacing = margin;
    layout.minimumLineSpacing = margin;
    CGFloat top = 44;
    if (iOS7Later){
      top += 20;
    }
  
    CGFloat collectionViewHeight = _imagePickerVc.maxImagesCount > 1 ? self.view.tz_height - 50 - top : self.view.tz_height - top;
    _collectionView = [[UICollectionView alloc] initWithFrame:CGRectMake(0, top, self.view.tz_width, collectionViewHeight) collectionViewLayout:layout];
    _collectionView.backgroundColor = [UIColor whiteColor];
    _collectionView.dataSource = self;
    _collectionView.delegate = self;
    _collectionView.alwaysBounceHorizontal = NO;
    _collectionView.contentInset = UIEdgeInsetsMake(margin, margin, margin, margin);
  
    _collectionView.contentSize = CGSizeMake(self.view.tz_width, ((_model.count + self.columnNumber - 1) / self.columnNumber) * self.view.tz_width);
  
    [self.view addSubview:_collectionView];
    [_collectionView registerClass:[TZAssetCell class] forCellWithReuseIdentifier:@"TZAssetCell"];
    [_collectionView registerClass:[TZAssetCameraCell class] forCellWithReuseIdentifier:@"TZAssetCameraCell"];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    //[self scrollCollectionViewToBottom];
    // Determine the size of the thumbnails to request from the PHCachingImageManager
    CGFloat scale = 2.0;
    if([UIScreen mainScreen].bounds.size.width > 600) {
        scale = 1.0;
    }
    CGSize cellSize = ((UICollectionViewFlowLayout *)_collectionView.collectionViewLayout).itemSize;
    AssetGridThumbnailSize = CGSizeMake(cellSize.width * scale, cellSize.height * scale);
}

#pragma mark - Click Event

- (void)cancel {
    if (_imagePickerVc.autoDismiss) {
        [self.navigationController dismissViewControllerAnimated:YES completion:nil];
    }
  
    if ([_imagePickerVc.pickerDelegate respondsToSelector:@selector(imagePickerControllerDidCancel:)]) {
        [_imagePickerVc.pickerDelegate imagePickerControllerDidCancel:_imagePickerVc];
    }
  
    if (_imagePickerVc.imagePickerControllerDidCancelHandle) {
        _imagePickerVc.imagePickerControllerDidCancelHandle();
    }
}

#pragma mark - UICollectionViewDataSource && Delegate

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return _models.count + 1;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    // the cell lead to take a picture / 去拍照的cell
    if (indexPath.row == 0) {
      TZAssetCameraCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"TZAssetCameraCell" forIndexPath:indexPath];
      cell.imageView.image = [UIImage imageNamedFromMyBundle:@"takePicture@2x.png"];
      return cell;
    }
  
    // the cell dipaly photo or video / 展示照片或视频的cell
    TZAssetCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"TZAssetCell" forIndexPath:indexPath];
    cell.photoDefImageName = _imagePickerVc.photoDefImageName;
    cell.photoSelImageName = _imagePickerVc.photoSelImageName;
  
    TZAssetModel *model = _models[indexPath.row - 1];
    cell.model = _models[indexPath.row - 1];
    cell.maxImagesCount = _imagePickerVc.maxImagesCount;
    
    __weak TZAssetCell* weakCell = cell;
    __weak TZPhotoPickerController* weakSelf = self;
    //__weak CALayer* weakLayer = _numberImageView.layer;
    cell.didSelectPhotoBlock = ^(BOOL isSelected) {
        // 1. cancel select / 取消选择
        if (isSelected) {
            weakCell.selectPhotoButton.selected = NO;
            model.isSelected = NO;
            NSArray *selectedModels = [NSArray arrayWithArray:_imagePickerVc.selectedModels];
            for (TZAssetModel *model_item in selectedModels) {
                if ([[[TZImageManager manager] getAssetIdentifier:model.asset] isEqualToString:[[TZImageManager manager] getAssetIdentifier:model_item.asset]]) {
                    [_imagePickerVc.selectedModels removeObject:model_item];
                    break;
                }
            }
          
            if(_refreshStatusDelegate){
              [_refreshStatusDelegate refreshBottomToolBarStatus];
            }
        } else {
            // 2. select:check if over the maxImagesCount / 选择照片,检查是否超过了最大个数的限制
            if (_imagePickerVc.selectedModels.count < _imagePickerVc.maxImagesCount) {
                weakCell.selectPhotoButton.selected = YES;
                model.isSelected = YES;
                [_imagePickerVc.selectedModels addObject:model];
                if(weakSelf.refreshStatusDelegate){
                  [weakSelf.refreshStatusDelegate refreshBottomToolBarStatus];
                }
            } else {
                NSString *title = [NSString stringWithFormat:@"最多支持%ld个选择", (long)_imagePickerVc.maxImagesCount];
              [FileBrowserPlugin showAlertWithTitle:self.navigationController title:title];
            }
        }
    };
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
  // take a photo / 去拍照
  if(indexPath.row == 0){
    [self takePhoto];
    return;
  }
}

#pragma mark - Private Method
- (void)takePhoto {
  AVAuthorizationStatus authStatus = [AVCaptureDevice authorizationStatusForMediaType:AVMediaTypeVideo];
  if ((authStatus == AVAuthorizationStatusRestricted || authStatus ==AVAuthorizationStatusDenied) && iOS7Later) {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
    // 无权限 做一个友好的提示
    NSString *appName = [[NSBundle mainBundle].infoDictionary valueForKey:@"CFBundleDisplayName"];
    if (!appName) appName = [[NSBundle mainBundle].infoDictionary valueForKey:@"CFBundleName"];
    NSString *message = [NSString stringWithFormat:@"请打开%@访问摄像头权限，通过\"设置 -> 隐私 -> 摄像头\"", appName];
    UIAlertView * alert = [[UIAlertView alloc]initWithTitle:@"访问摄像头受限" message:message delegate:self cancelButtonTitle:@"取消" otherButtonTitles:@"设置", nil];
    [alert show];
#pragma clang diagnostic pop
  } else { // 调用相机
    UIImagePickerController *picker = [[UIImagePickerController alloc] init];
    if([UIImagePickerController isSourceTypeAvailable:UIImagePickerControllerSourceTypeCamera]) {
      picker.sourceType = UIImagePickerControllerSourceTypeCamera;
      picker.mediaTypes = [UIImagePickerController availableMediaTypesForSourceType:UIImagePickerControllerSourceTypeCamera];
      //Camera所支持的Media格式都有哪些,共有两个分别是@"public.image",@"public.movie"
      picker.videoMaximumDuration = 30.0f;//30秒
      picker.delegate = self;
      if(iOS8Later) {
        picker.modalPresentationStyle = UIModalPresentationOverCurrentContext;
      }
      [self presentViewController:picker animated:YES completion:nil];
    }else {
      NSLog(@"模拟器中无法打开照相机,请在真机中使用");
    }
  }
}

#pragma mark - UIImagePickerControllerDelegate

- (void)imagePickerController:(UIImagePickerController*)picker didFinishPickingMediaWithInfo:(NSDictionary *)info {
  
  NSString *type = [info objectForKey:UIImagePickerControllerMediaType];
  if ([type isEqualToString:@"public.image"]) {
    [FileBrowserPlugin showProgressHUD:60];
    UIImage *image = [info objectForKey:UIImagePickerControllerOriginalImage];
    [[TZImageManager manager] savePhotoWithImage:image completion:^(NSError *error){
      if(!error) {
        [self reloadPhotoArray];
        [FileBrowserPlugin hideProgressHUD];
      }
    }];
    [picker dismissViewControllerAnimated:YES completion:nil];
    return;
  }
  
  if([type isEqualToString:@"public.movie"]){
    [FileBrowserPlugin showProgressHUD:360];
    NSString* path = [[info objectForKey:UIImagePickerControllerMediaURL] path];
    UISaveVideoAtPathToSavedPhotosAlbum(path, self, @selector(video:didFinishSavingWithError:contextInfo:), nil);
    [picker dismissViewControllerAnimated:YES completion:nil];
  }
}

// 视频保存回调
#pragma mark 视频保存完毕的回调
- (void)video:(NSString *)videoPath didFinishSavingWithError:(NSError *)error contextInfo:(void *)contextInf{
  if(error){
    [FileBrowserPlugin hideProgressHUD];
    NSString *title = [NSString stringWithFormat:@"保存视频错误，错误信息:%@",error.localizedDescription];
    [FileBrowserPlugin showAlertWithTitle:self.navigationController title:title];
  }else{
    NSLog(@"视频保存成功.");
    NSLog(@"%@",videoPath);
    [self reloadPhotoArray];
    [FileBrowserPlugin hideProgressHUD];
  }
}

- (void)scrollCollectionViewToBottom {
  if (_shouldScrollToBottom && _models.count > 0 && _imagePickerVc.sortAscendingByModificationDate) {
    NSInteger item = _models.count - 1;
    [_collectionView scrollToItemAtIndexPath:[NSIndexPath indexPathForItem:item inSection:0] atScrollPosition:UICollectionViewScrollPositionBottom animated:NO];
    _shouldScrollToBottom = NO;
  }
}

- (void)reloadPhotoArray {
  [[TZImageManager manager] getCameraRollAlbum:_imagePickerVc.allowPickingVideo allowPickingImage:_imagePickerVc.allowPickingImage completion:^(TZAlbumModel *model) {
    _model = model;
    [[TZImageManager manager] getAssetsFromFetchResult:_model.result allowPickingVideo:_imagePickerVc.allowPickingVideo allowPickingImage:_imagePickerVc.allowPickingImage completion:^(NSArray<TZAssetModel *> *models) {
      
      TZAssetModel *assetModel;
      if (_imagePickerVc.sortAscendingByModificationDate) {
        assetModel = [models lastObject];
        [_models addObject:assetModel];
      } else {
        assetModel = [models firstObject];
        [_models insertObject:assetModel atIndex:0];
      }
      
      if (_imagePickerVc.maxImagesCount <= 1) {
        [_imagePickerVc.selectedModels addObject:assetModel];
        [_imagePickerVc okButtonClick];
        return;
      }
      
      if (_imagePickerVc.selectedModels.count < _imagePickerVc.maxImagesCount) {
        assetModel.isSelected = YES;
        [_imagePickerVc.selectedModels addObject:assetModel];
        if(self.refreshStatusDelegate){
          [self.refreshStatusDelegate refreshBottomToolBarStatus];
        }
      }
      [_collectionView reloadData];
      
      _shouldScrollToBottom = YES;
      [self scrollCollectionViewToBottom];
    }];
  }];
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker {
  [picker dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - UIAlertViewDelegate

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
#pragma clang diagnostic pop
  if (buttonIndex == 1) { // 去设置界面，开启相机访问权限
    if (iOS8Later) {
      [[UIApplication sharedApplication] openURL:[NSURL URLWithString:UIApplicationOpenSettingsURLString]];
    } else {
      [[UIApplication sharedApplication] openURL:[NSURL URLWithString:@"prefs:root=Privacy&path=CAMERA"]];
    }
  }
}

- (void)checkSelectedModels {
    for (TZAssetModel *model in _models) {
        model.isSelected = NO;
        NSMutableArray *selectedAssets = [NSMutableArray array];
        for (TZAssetModel *model in _imagePickerVc.selectedModels) {
            [selectedAssets addObject:model.asset];
        }
      
        if ([[TZImageManager manager] isAssetsArray:selectedAssets containAsset:model.asset]) {
            model.isSelected = YES;
        }
    }
}

@end
